# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk
import os

import Theme
import htmltextview
import desktop
from Widgets import WidgetToggleBox
from emesenecommon import * # no paths here

class PreferenceWindow(gtk.Window):
    '''This class display a windows where the user
    can modify the behaviour of emesene'''
    
    def __init__(self, controller, config, parent):
        '''Contructor'''
        gtk.Window.__init__(self)
        
        self.controller = controller
        self.set_default_size(150, 150)
        self.set_title(_('Preferences'))
        self.set_role(_('Preferences'))

        self.config = config
        self.set_transient_for(parent)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        
        self.set_border_width(6)
        vbox = gtk.VBox()
        vbox.set_spacing(4)
        hButBox = gtk.HButtonBox()
        hButBox.set_spacing(4)
        hButBox.set_layout(gtk.BUTTONBOX_END)
        notebook = gtk.Notebook()
        notebook.set_scrollable(True)
        notebook.set_border_width(3)
        bClose = gtk.Button(stock=gtk.STOCK_CLOSE)
        
        vbox.pack_start(notebook, True, True)
        hButBox.pack_start(bClose)
        vbox.pack_start(hButBox, False, False)
        
        if self.controller.connected:
            self.interface_tab = InterfaceTab(self.controller)
            self.themesTab = ThemesTab(self.controller)
            self.colorSchemeTab = ColorSchemeTab(self.controller)
            self.ftTab = FTSettings(self.controller)
            notebook.append_page(self.themesTab, gtk.Label(_('Theme')))
            notebook.append_page(self.interface_tab, gtk.Label(_('Interface')))
            notebook.append_page(self.colorSchemeTab,
                gtk.Label(_('Color scheme')))
            notebook.append_page(self.ftTab,
                gtk.Label(_('Filetransfer')))

        self.desktopTab = DesktopTab(self.controller)
        self.connectionTab = ConnectionTab(self.config)

        if self.controller.connected:
            notebook.append_page(self.desktopTab, gtk.Label(_('Desktop')))
            notebook.append_page(self.connectionTab, gtk.Label(_('Connection')))
        else:
            notebook.append_page(self.connectionTab, gtk.Label(_('Connection')))
            notebook.append_page(self.desktopTab, gtk.Label(_('Desktop')))

        
        vbox.show_all()
        self.add(vbox)
        
        self.connect('delete-event', self.close)
        bClose.connect('clicked', self.close)

    def close(self, *args):
        '''Close the window'''
        
        self.hide()
        self.save()
        
        self.destroy()
        self.controller.preference_open = False
        
    def save(self):
        '''save the actual setting'''

        self.connectionTab.save()
        
        if self.controller.connected:
            self.interface_tab.save()
            self.themesTab.save()
            self.colorSchemeTab.save()
            self.desktopTab.save()
            self.ftTab.save()
            # TODO: remove preferences-changed
            self.controller.emit('preferences-changed')

class ColorSchemeItem:
    def __init__(self, tab, configColor,text):
        self.configColor=configColor
        self.button = gtk.ColorButton(gtk.gdk.color_parse(tab.config.user[configColor]))
        self.button.connect("color-set", tab.save)
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(text))
        self.hbox = gtk.HBox()
        self.hbox.pack_start(alignment, True, True, 3)
        self.hbox.pack_start(self.button, False, False, 3)

class ColorSchemeTab(gtk.VBox):
    '''This class represent the tab that contain all the configuration about the colors'''
    
    def __init__(self, controller):
        gtk.VBox.__init__(self)
        
        self.config = controller.config
        self.controller = controller
        
        self.items={}
        self.items["typingColor"] = ColorSchemeItem(self, 'typingColor', \
            _("Contact typing"))
        self.items["messageWaitingColor"] = ColorSchemeItem(self, \
            'messageWaitingColor', _("Message waiting"))
        self.items["personalMessageColor"] = ColorSchemeItem(self, \
            'personalMessageColor', _("Personal msn message"))
        
        vbox = gtk.VBox()
        
        for i in self.items.keys():
            vbox.pack_start(self.items[i].hbox, False, False, 3)
        
        #scroll = gtk.ScrolledWindow()
        #scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        #scroll.add_with_viewport(vbox)
        
        #self.pack_start(scroll, True, True, 0)
        self.pack_start(vbox, True, True, 0)
        self.pack_start(gtk.HSeparator(), False, True, 2)
        
        restoreButton = gtk.Button(stock = gtk.STOCK_CLEAR)
        restoreButton.connect("clicked", self.restoreColors)
        
        alignment = gtk.Alignment(0.5, 0.5, 0.5, 0.5)
        alignment.add(restoreButton)
        self.pack_start(alignment, False, True, 2)
                        
        self.show_all()
    
    def restoreColors(self, *args):
        for i in self.items.keys():
            color = gtk.gdk.color_parse(self.config.user.getDefault(self.items[i].configColor))
            self.items[i].button.set_color(color) 
    
    def save(self, *args):
        for i in self.items.keys():
            self.config.user[self.items[i].configColor] = rgbToHexa(self.items[i].button.get_color())
        

class DesktopTab(gtk.VBox):
    '''A tab that lets the user override the detected desktop settings'''

    def __init__(self, controller):
        gtk.VBox.__init__(self)
        
        self.config = controller.config
        
        self.urlSettings = UrlSettings(controller.config)
        frame = gtk.Frame(_('Links and files'))
        frame.set_border_width(4)
        frame.add(self.urlSettings)
        
        self.set_spacing(8)
        self.set_border_width(8)
        
        self.rgba = gtk.CheckButton(_('Enable rgba colormap (requires restart)'))
        self.rgba.set_active(self.config.glob['rgbaColormap'])
        
        self.pack_start(self.rgba, False)
        self.pack_start(frame, False)

    def save(self, widget=None):
        self.config.glob['rgbaColormap'] = self.rgba.get_active()
        self.urlSettings.save()

            
class UrlSettings(gtk.VBox):
    def __init__(self, config):
        gtk.VBox.__init__(self)
        
        self.set_spacing(8)
        self.set_border_width(8)
        
        self.config = config
        
        detected = desktop.get_desktop(True)
        if detected:
            commandline = ' '.join(desktop.get_command(detected, '')).strip()
            tmp = {
                'detected': detected,
                'commandline': commandline,
            }
            markup = _('The detected desktop environment is '
                '<b>"%(detected)s"</b>.\n'
                '<span face="Monospace">%(commandline)s</span> '
                'will be used to open links and files') % tmp
        else:
            markup = _('<b>No desktop environment detected.</b> '
                'The first browser found will be used to open links')
        
        self.infolabel = gtk.Label()
        self.infolabel.set_markup(markup)
        self.infolabel.set_line_wrap(True)
        self.infolabel.set_alignment(0.0, 0.0)
        
        self.hboxentry = gtk.HBox()
        self.entry = gtk.Entry()
        self.entry.connect('activate', self.save)
        self.hboxentry.pack_start(gtk.Label(_("Command line:")), False)
        self.hboxentry.pack_start(self.entry)
        
        self.override = gtk.CheckButton(_('Override detected settings'))
        self.override.set_active(self.config.glob['overrideDesktop'] != '')
        self.override.connect('toggled', self.toggleOverride)

        self.helplabel = gtk.Label()
        self.helplabel.set_markup(_("<i>Note:</i> %s is replaced by "
            "the actual url to be opened") % "%url%")
        self.helplabel.set_alignment(0.5, 1.0)
        
        self.hboxtest = gtk.HBox()
        self.testbutton = gtk.Button(_('Click to test'))
        self.testbutton.connect('clicked', self.testDesktop)
        self.hboxtest.pack_start(gtk.Label())
        self.hboxtest.pack_start(self.testbutton, False, True, 6)
        
        self.pack_start(self.infolabel, False)
        self.pack_start(self.override, False)
        self.pack_start(self.hboxentry, False)
        self.pack_start(self.hboxtest, False)
        self.pack_start(self.helplabel, False)
        
        self.toggleOverride()

    def toggleOverride(self, override=None):
        active = self.override.get_active()
        self.hboxentry.set_sensitive(active)
        self.hboxtest.set_sensitive(active)
        if active:
            self.entry.set_text(self.config.glob['overrideDesktop'])
        else:
            self.entry.set_text('')
        self.save()
        
        
    def save(self, widget=None):
        desktop.override = self.entry.get_text()
        self.config.glob['overrideDesktop'] = self.entry.get_text()        

    def testDesktop(self, button):
        self.save()
        try:
            desktop.open('http://www.emesene.org')
        except OSError:
            pass               

class ConnectionTab(gtk.VBox):
    '''This class represent the tab that contains the connection options'''
    
    def __init__(self, config):
        '''Constructor'''
        
        gtk.VBox.__init__(self)

        self.config = config

        self.set_spacing(8)
        self.set_border_width(8) 

        self.debug = gtk.CheckButton(_('_Show debug in console'))
        self.debug.set_active(self.config.glob['debug'])
        self.binary = gtk.CheckButton(_('_Show binary codes in debug'))
        self.binary.set_active(self.config.glob['binary'])
        self.httpMethod =  gtk.CheckButton(_('_Use HTTP method'))
        self.httpMethod.set_active(self.config.glob['httpMethod'])
        
        self.proxySettings = ProxySettings(self.config)
        frame = gtk.Frame(_('Proxy settings'))
        frame.set_border_width(4)
        frame.add(self.proxySettings)

        self.pack_start(self.debug, False, False)
        self.pack_start(self.binary, False, False)
        self.pack_start(self.httpMethod, False, False)
        
        proxyBox = gtk.VBox(spacing=2)
        proxyBox.set_border_width(4)
        proxyBox.pack_start(frame, False, False)
        
        self.pack_start(proxyBox, True, True)

        self.show_all()
    
    def save(self):
        '''save the actual setting'''
    
        self.config.glob['debug'] = self.debug.get_active()
        self.config.glob['binary'] = self.binary.get_active()
        self.config.glob['httpMethod'] = self.httpMethod.get_active()
        self.proxySettings.save()
        

class ThemesTab(gtk.VBox):
    '''This class represent the tab that ...'''
    
    def __init__(self, controller):
        '''Constructor'''
        
        gtk.VBox.__init__(self)
        
        self.controller = controller
        self.config = controller.config
        self.theme = controller.theme
        self.basetheme = Theme.BaseTheme()
        self.basetheme.pixbufs = {}
        self.basetheme.defaults = {}
        self.basetheme.HOME_PATH = self.theme.HOME_PATH
        self.basetheme.SYSTEM_WIDE_PATH = self.theme.SYSTEM_WIDE_PATH
        self.basetheme.defaultPath = self.theme.defaultPath
        
        self.set_spacing(8)
        self.set_border_width(8)
        
        self.smallIcons = gtk.CheckButton(_('_Use small icons'))
        self.smallIcons.set_active(self.config.user['smallIcons'])
        self.parseSmilies = gtk.CheckButton(_('Display _smiles'))
        self.parseSmilies.set_active(self.config.user['parseSmilies'])
        self.disableFormat = gtk.CheckButton(_('Disable text formatting'))
        self.disableFormat.set_active(self.config.user['disableFormat'])
        
        self.pack_start(self.smallIcons, False, False)
        self.pack_start(self.parseSmilies, False, False)
        self.pack_start(self.disableFormat, False, False)

        fields = []
        
        fields.append({'name': 'theme',
                       'label': _('Theme'),
                       'values': self.config.getThemes(),
                       'config': None,
                       'sample': self.getThemeSample,
                       })
        
        fields.append({'name': 'smilieTheme',
                       'label': _('Smilie theme'),
                       'values': self.config.getSmilieThemes(),
                       'config': None,
                       'sample': self.getSmilieSample,
                       })
        
        fields.append({'name': 'conversationLayout',
                       'label': _('Conversation Layout'),
                       'values': self.controller.conversationLayoutManager.listAvailableThemes(),
                       'config': self.configConversationLayout,
                       'sample': None,
                       })
        
        self.fields = fields
        
        table = gtk.Table(len(fields), 2, False)
        table.set_row_spacings(2)
        table.set_col_spacings(2)
        row = 0
        for field in fields:
            label = gtk.Label(field['label'])
            label.set_alignment(0.0, 0.5)
            label.set_use_underline(True)
            table.attach(label, 0,1, row,row+1)
            
            comboModel = gtk.ListStore(str, gtk.gdk.Pixbuf)
            combo = gtk.ComboBox(comboModel)
            cellRenderer = gtk.CellRendererText()
            sampleImage = gtk.CellRendererPixbuf()
            sampleImage.set_property('xalign', 1.0)

            combo.pack_start(cellRenderer, False)
            combo.add_attribute(cellRenderer, 'markup', 0)
            combo.pack_start(sampleImage, True)
            combo.add_attribute(sampleImage, 'pixbuf', 1)

            self.fields[row]['combo'] = combo
            label.set_mnemonic_widget(combo)
            combo.connect('changed', self.save, field['name'])
            
            default = self.config.user[field['name']]
            done = False
            count = 0
            for current in field['values']:
                sample = None
                if field['sample']:
                    sample = field['sample'](field['name'], current)
                if not done:
                    if current == default: done = True
                    else: count += 1
                comboModel.append([ current, sample ])
            combo.set_active(count)
            table.attach(combo, 1,2, row,row+1)
            
            #button = gtk.Button(_('Edit...'))
            
            #if field['config'] != None:
            #    table.attach(button, 2,3, row,row+1)
            #    button.connect('clicked', field['config'])
            #else:
            #    button.set_sensitive(False)
            
            row += 1
        self.pack_start(table, False, False, 3)
        self.show_all()
        
    # Config dialogs
    def configConversationLayout(self, *args):
        dialog = gtk.Dialog(_("Conversation Layout"),
                             None,
                             gtk.DIALOG_MODAL,
                             (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        conversationLayoutTab = ConversationLayoutTab(self.controller)
        dialog.vbox.add(conversationLayoutTab)
        dialog.show_all()
        response = dialog.run()
        dialog.destroy()
        # do something with the changes here
    
    # Samples
    def getThemeSample(self, label, item):
        self.basetheme.setTheme(item)
        return self.basetheme.getImage("icon")
    
    def getSmilieSample(self, label, item):
        tempTheme = Theme.SmilieTheme(None, item)
        try:
            icon = tempTheme.getSmiley(":D").get_static_image()
            del tempTheme
            return icon
        except:
            pass
    
    # Other functions
    def save(self, widget = None, name = 'all'):
        self.config.user['parseSmilies'] = self.parseSmilies.get_active()
        self.config.user['smallIcons'] = self.smallIcons.get_active()
        self.config.user['disableFormat'] = self.disableFormat.get_active()
        try:
            for field in self.fields:
                self.config.user[field['name']] = field['combo'].get_active_text()
        except:
            pass
        
        if name in ('all','conversationLayout'): 
            layout = self.config.user['conversationLayout']
            self.controller.conversationLayoutManager.load(layout)

#TODO: this is not a tab
class ConversationLayoutTab(gtk.VBox):
    '''This class represents the conversation layout config'''
    
    def __init__(self, controller):
        '''Constructor'''
        gtk.VBox.__init__(self)
        
        self.controller = controller
        self.config = controller.config
        
        self.set_spacing(8)
        self.set_border_width(8) 
        
        # Preview field
        self.labelName = gtk.Label()
        self.labelName.set_alignment(0.0, 0.5)
        self.labelDescription = gtk.Label()
        self.labelDescription.set_alignment(0.0, 0.5)
        self.labelAuthor = gtk.Label()
        self.labelAuthor.set_alignment(0.0, 0.5)
        self.labelWebsite = gtk.Label()
        self.labelWebsite.set_alignment(0.0, 0.5)
        self.previewScroll = gtk.ScrolledWindow()
        self.previewScroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.preview = htmltextview.HtmlTextView(self.controller)
        self.previewScroll.add(self.preview)
        self.pack_start(self.labelName, False, False, 3)
        self.pack_start(self.labelDescription, False, False, 3)
        self.pack_start(self.labelAuthor, False, False, 3)
        self.pack_start(self.labelWebsite, False, False, 3)
        self.pack_start(self.previewScroll, True, True, 3)
        self.updateInfosFrame()

        self.show_all()
        
    def updateInfosFrame(self):
        '''Updates the htmltextview preview.'''
        mgr = self.controller.conversationLayoutManager
        self.labelName.set_markup('<b>' + _('Name:') + '</b> ' + \
            mgr.getName())
        self.labelDescription.set_markup('<b>' + _('Description:') + '</b> ' +\
            mgr.getDescription())
        self.labelAuthor.set_markup('<b>' + _('Author:') + '</b> ' + \
            mgr.getAuthor())
        self.labelWebsite.set_markup('<b>' + _('Website:') + '</b> ' + \
            mgr.getWebsite())
        
        textbuff = self.preview.get_buffer()
        start, end = textbuff.get_bounds()
        textbuff.delete(start,end)
 
        html = mgr.getPreview()  
        BODY = "<body>%s</body>"
        self.preview.display_html(BODY % html)

class InterfaceTab(gtk.VBox):
    '''This class represents the [...]'''
    
    def __init__(self, controller):
        '''Constructor'''
        
        gtk.VBox.__init__(self)
        
        self.controller = controller
        self.config = controller.config
        
        self.set_spacing(2)
        self.set_border_width(8)
        
        self.label = gtk.Label()

        def add(box, key, desc, size, packing=False, align=None, defsize=-1):
            widget = WidgetToggleBox(self.config, key, desc, self.label)
            widget.show()
            if type(box) == gtk.HBox:
                widget.set_size_request(size, defsize)
            else:
                widget.set_size_request(defsize, size)
            if align:
                align.add(widget)
                widget = align
            box.pack_start(widget, packing)
        
        self.windowshbox = gtk.HBox()

        self.maindescvbox = gtk.VBox()
        self.mainalign = gtk.Alignment(0.5, 0.5, 0.0, 0.0)
        self.mainvbox = gtk.VBox()
        self.mainvbox.set_size_request(60, -1)
        add(self.mainvbox, 'showMenu', _('Show _menu bar'), 10)
        add(self.mainvbox, 'showUserPanel', _('Show _user panel'), 20)
        add(self.mainvbox, 'showSearchEntry', _('Show _search entry'), 10)
        add(self.mainvbox, None, None, 80)
        add(self.mainvbox, 'showStatusCombo', _('Show status _combo'), 10)
        
        mainwindowlabel = gtk.Label()
        mainwindowlabel.set_markup('<b>' + _('Main window') + '</b>')
        mainwindowlabel.set_alignment(0.5, 0.0)
        self.mainalign.add(self.mainvbox)
        self.maindescvbox.pack_start(mainwindowlabel)
        self.maindescvbox.pack_start(self.mainalign, False)
        self.windowshbox.pack_start(self.maindescvbox)
        
        self.convdescvbox = gtk.VBox()
        self.convalign = gtk.Alignment(0.5, 0.5, 0.0, 0.0)
        self.convbox1 = gtk.VBox()
        self.convbox1.set_size_request(100, -1)
        add(self.convbox1, 'showMenubar', _('Show _menu bar'), 10)
        add(self.convbox1, 'showHeader', _('Show conversation _header'), 20)
        self.convbox2 = gtk.HBox()
        self.convbox3 = gtk.VBox()
        self.convbox4 = gtk.HBox()
        self.convbox5 = gtk.VBox()
        self.convbox1.pack_start(self.convbox2)
        self.convbox2.pack_start(self.convbox3)
        add(self.convbox3, None, None, 50)
        self.convbox3.pack_start(self.convbox4)
        self.convbox4.pack_start(self.convbox5)
        self.convbox4.set_size_request(-1, 30)
        add(self.convbox5, 'showToolbar', _('Show conversation _toolbar'), 10)
        add(self.convbox5, None, None, -1, packing=True)
        add(self.convbox4, 'showSendButton', _('S_how a Send button'), 15,
            align=gtk.Alignment(0.0, 0.5, 0.0, 0.0), defsize=10)

        add(self.convbox3, 'showStatusBar', _('Show st_atusbar'), 10)
        add(self.convbox2, 'showAvatars', _('Show a_vatars'), 30)
        
        convwindowlabel = gtk.Label()
        convwindowlabel.set_markup('<b>' + _('Conversation Window') + '</b>')
        convwindowlabel.set_alignment(0.5, 0.0)
        self.convalign.add(self.convbox1)
        self.convdescvbox.pack_start(convwindowlabel)
        self.convdescvbox.pack_start(self.convalign, True)
        self.windowshbox.pack_start(self.convdescvbox)
        
        self.pack_start(self.windowshbox)

        self.pack_start(self.label, True)

        self.checks = []
        mkcheck(self, 'windows', _('Use multiple _windows'))
        mkcheck(self, 'showTabCloseButton', _('Show close button on _each tab'))
        mkcheck(self, 'avatarsInUserList', _('Show _avatars'))
        mkcheck(self, 'avatarsInTaskbar', _('Show avatars in task_bar'))
        #mkcheck(self, 'showMailTyping', _('Show ma_il on "is typing" message'))
        #mkcheck(self, 'disableEsc', _('Don\'t _close by pressing ESC'))
        
        self.show_all()

            
    def onToggled(self, radio, option):
        self.config.user[option] = radio.get_active()
        
    def save(self):
        pass

class ProxySettings(gtk.VBox):
    '''This class represents the panel with the proxy variables
    in the config file'''

    def __init__(self, config):
        '''Constructor'''
        
        gtk.VBox.__init__(self)   

        self.config = config
        
        self.set_spacing(8)
        self.set_border_width(8)

        self.useProxy = gtk.CheckButton(_('_Use proxy'))
        self.useProxy.set_active(self.config.glob['useProxy'])

        self.host = gtk.Entry()
        self.host.set_text(self.config.glob['proxyHost'])
        self.port = gtk.Entry()
        self.port.set_text(str(self.config.glob['proxyPort']))

        table = gtk.Table(2, 2)
        table.set_row_spacings(2)
        table.set_col_spacings(2)
        
        host = gtk.Label(_('_Host:'))
        host.set_alignment(0.0, 0.5)
        host.set_use_underline(True)
        host.set_mnemonic_widget(self.host)
        port = gtk.Label(_('_Port:'))
        port.set_alignment(0.0, 0.5)
        port.set_use_underline(True)
        port.set_mnemonic_widget(self.port)
        table.attach(host, 0, 1, 0, 1)
        table.attach(port , 0, 1, 1, 2)
        
        table.attach(self.host, 1, 2, 0, 1)
        table.attach(self.port, 1, 2, 1, 2)
        self.useProxyToggled(self.useProxy)
        self.useProxy.connect('toggled', self.useProxyToggled)
        self.pack_start(self.useProxy)
        self.pack_start(table)


        self.show_all()
    
    def save(self):
        '''save the actual setting'''
        
        host = self.host.get_text()
        if host.startswith("http://"):
            host = host.split("http://")[1]
        if host.find("/") != -1:
            host = host.split("/")[0]

        self.config.glob['useProxy'] = self.useProxy.get_active()
        self.config.glob['proxyHost'] = host
        self.config.glob['proxyPort'] = self.port.get_text()

    def useProxyToggled(self, check):
        '''callback for the toggled signal'''
        
        if self.useProxy.get_active():
            self.host.set_sensitive(True)
            self.port.set_sensitive(True)
        else:
            self.host.set_sensitive(False)
            self.port.set_sensitive(False)

class FTSettings(gtk.VBox):
    '''This class represents the panel with the filetransfer settings'''
    
    def __init__(self, controller):
        '''Constructor'''
        
        gtk.VBox.__init__(self)
        
        self.config = controller.config
        self.pathChooser = gtk.FileChooserDialog(
            title=_('Choose a Directory'),
            action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            buttons=(
                gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
                gtk.STOCK_OPEN, gtk.RESPONSE_OK)
            )
        
        self.set_spacing(8)
        self.set_border_width(8)
        
        self.checks = []
        mkcheck(self, 'receivedFilesSortedByUser',
            _('Sort received _files by sender'))
        
        targetbutton = gtk.FileChooserButton(self.pathChooser)
        targetbutton.set_current_folder(os.path.expanduser(
            self.config.user['receivedFilesDir']))

        target = gtk.Label(_('_Save files to:'))
        target.set_alignment(0.0, 0.5)
        target.set_use_underline(True)
        self.pack_start(target, False)
        self.pack_start(targetbutton, False)
        
    def onToggled(self, radio, option):
        self.config.user[option] = radio.get_active()
        
    def save(self):
        self.config.user['receivedFilesDir'] = self.pathChooser.get_filename()

def mkcheck(self, id, label):
    '''little helper function to add checkbuttons'''
    check = gtk.CheckButton(label)
    check.set_active(self.config.user[id])
    check.connect('toggled', self.onToggled, id)
    self.pack_start(check, False, False, padding=0)
    setattr(self, id, check)
    self.checks.append(id)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'
from urllib import urlopen, urlencode
import Plugin

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = 'Create tiny url with /tiny <url>'
        self.authors = { 'Roger Duran' : 'rogerduran@gmail.com' }
        self.website = 'http://www.rogerpc.com.ar'
        self.displayName = 'Tiny Url'
        self.name = 'TinyUrl'
        self.controller = controller
        self.Slash = controller.Slash

    def start( self ):
        '''start the plugin'''
        self.Slash.register('tiny', self.get_tiny, 'Create a tiny url')
        self.enabled = True

    def get_tiny( self, slash_action ):
        '''Return Tiny Url'''
        data = slash_action.getParams()
        
        params = urlencode({'url': data})
        page = urlopen('http://tinyurl.com/create.php',params).read()
        
        url = page.split('[<a href="')[1].split('" ')[0]
        slash_action.outputText( url, True )

    def stop( self ):    
        '''stop the plugin'''
        self.Slash.unregister('tiny')
        self.enabled = False

    def check( self ):
        '''Check Plugin'''
        return ( True, 'Ok' )




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import common

import sys
import Queue
import threading
import httplib

requestQueue = Queue.Queue( 0 )
responseQueue = Queue.Queue( 0 )

def put(request):
    global requestQueue
    #common.debug("SoapManager.put(%s%s)" % (request.host, request.path), "soap")
    requestQueue.put(request)
    
def get():
    global responseQueue
    retval = responseQueue.get()
    common.debug("SoapManager.get() --> %s%s" % (retval.host, retval.path),
        "soap")
    return retval

def getNoBlock():
    global responseQueue
    
    return responseQueue.get(True, 0.01)

def process():
    '''run the callbacks if some response is in responseQueue'''
    global responseQueue
    
    while True:
        try:
            response = getNoBlock()
            common.debug("SoapManager.process: %s%s" %
                (response.host, response.path), "soap")
                        
            if response.callback != None:
                response.callback( response, *response.args )
        except Queue.Empty:
            break

    return True

class SoapRequest( object ):
    '''This class represent a soapRequest'''
    
    def __init__( self, action, host, port, path, body, \
                  callback=None, args=() ):
        '''Constructor'''
        
        self.action = action
        self.host = host
        self.port = port
        self.path = path
        self.body = body
        self.callback = callback
        self.args = args
        self.extraCookie = ''
        self.errorCount = 0
        self.status = ( 0 , '' )

class SoapManager( threading.Thread ):
    '''This class is a thread that wait for soapRequest in the requestQueue
    and make the request, then add the response to the responseQueue'''
       
    def __init__( self, msn ):
        threading.Thread.__init__( self )
        
        self.msn = msn

    def process( self ):
        try:
            return process()
        except:
            self.msn.emit('exception', sys.exc_info())

    def destroy( self ):
        global requestQueue, responseQueue
        try:
            while True:
                tmp = responseQueue.get(True, 0.01)
                for i in dir(tmp):
                    if not i.startswith('_'):
                        setattr(tmp, i, None)
        except Queue.Empty:
            pass

        try:
            while True:
                tmp = requestQueue.get(True, 0.01)
                for i in dir(tmp):
                    if not i.startswith('_'):
                        setattr(tmp, i, None)
        except Queue.Empty:
            pass
        
        put('quit')
        self.msn = None

    def run( self ):
        global requestQueue, responseQueue
        
        while True:
            req = requestQueue.get()
            
            if req == 'quit' or self.msn == None:
                break
            else:
                try:
                    responseQueue.put( self.makeSoapRequest( req ) )
                except Exception, e:
                    print e
                    if req.errorCount < 2:
                        req.errorCount += 1
                        requestQueue.put( req )
                
        return False
        
    def makeSoapRequest( self, soapRequest, retry=True ):
    
        headers = { \
            "SOAPAction": soapRequest.action, \
            "Content-Type": "text/xml; charset=utf-8", \
            "Cookie": "MSPAuth=" + self.msn.MSPAuth + ";MSPProf=" + \
                self.msn.MSPProf + soapRequest.extraCookie, \
            "Host": soapRequest.host, \
            "Content-Length": str( len(soapRequest.body) ), \
            "User-Agent": "MSN Explorer/9.0 (MSN 8.0; TmstmpExt)", \
            "Connection": "Keep-Alive", \
            "Cache-Control": "no-cache", \
        }
        
        conn = None
        response = None
        
        if soapRequest.port == 443:
            conn = httplib.HTTPSConnection( soapRequest.host + ':' + \
                str(soapRequest.port) )  
        else:
            conn = httplib.HTTPConnection( soapRequest.host + ':' + \
                str(soapRequest.port) ) 
             
        conn.request( "POST", soapRequest.path, soapRequest.body, headers )
        response = conn.getresponse()

        soapResponse = SoapRequest( soapRequest.action, soapRequest.host, \
            soapRequest.port, soapRequest.path, response.read(), \
            soapRequest.callback, soapRequest.args )            
        soapResponse.status = ( response.status, response.reason )
        
        if soapResponse.body.count('TweenerChallenge') or \
                          soapResponse.body.count('LockKeyChallenge'):
            retry = False

        if retry and soapResponse.body.count('AuthenticationFailure') or \
                                 soapResponse.body.count('PassportAuthFail'):
            self.msn.passportReAuth()
            soapResponse = self.makeSoapRequest( soapRequest, False )
        
        return soapResponse 



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk

disabled = False
type_ = 'gtk'

if not gtk.check_version( 2, 10, 0 ) == None:
    if os.name == 'posix':
        try:
            import egg.trayicon
            type_ = 'egg'
            disabled = False
        except:
            print 'No tray icon library detected'
            disabled = True
    elif os.name == 'nt':
        try:
            from gtkwin32 import *
            WM_LBUTTONUP = 0x0202
            WM_RBUTTONUP = 0x0205
            type_ = 'win'
            disabled = False
        except:
            print 'No tray icon library detected'
            disabled = True
        
class TrayIcon:
    '''This class creates the tray icon notification - Pre-GTK 2.10'''

    def __init__(self, controller):
        '''Constructor'''
        global disabled
        
        self.controller = controller
        self.config = self.controller.config
        self.mainWindow = self.controller.mainWindow
        self.theme = self.controller.theme
        self.status = ''
        
        try:
            if disabled:
                pass
            elif type_ == 'gtk':
                self.tray = gtk.StatusIcon()
                self.tray.set_tooltip( 'emesene' )
                pixbuf = self.theme.getImage('trayicon')
                self.tray.set_from_pixbuf( pixbuf )
                self.buildMenu()
                self.tray.hide = lambda: self.tray.set_visible( False )
                self.tray.show = lambda: self.tray.set_visible( True )
                
                self.tray.connect( 'activate', self.on_activate )
                self.tray.connect( 'popup-menu', self.on_popup_menu )
            elif os.name == 'posix':
                self.tray = egg.trayicon.TrayIcon('emesene')
                self.buildTrayIconPosix()
                self.buildMenu()
            elif os.name == 'nt':
                # Note: gtk window must be realized before installing extensions.
                self.mainWindow.realize()
                self.win32ext = GTKWin32Ext(self.mainWindow)
                self.buildMenu()
                self.buildTrayIconWin32()
        except Exception, e:
            print 'exception creating trayicon: ' + str( e )
            disabled = True
            
    def remove(self):
        '''remove the trayicon'''

        if os.name == 'nt': 
            self.win32ext.remove_notify_icon()
        
    def buildTrayIconPosix(self):
        '''Build the trayIcon for linux'''

        self.eventBox = gtk.EventBox()
        pixbuf = self.theme.getImage('trayicon')
        self.image = gtk.Image()
        self.image.set_from_pixbuf( pixbuf )

        self.eventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.eventBox.connect_object('button_press_event', self.iconClickPosix, self.eventBox)

        self.tooltips = gtk.Tooltips()
        self.tooltips.set_tip(self.eventBox, 'Emesene')
        
        self.eventBox.add(self.image)
        self.tray.add(self.eventBox)
        self.eventBox.show_all()
        self.tray.show_all()

    def update(self, newUserStatus, pixbuf=None):
        
        self.status = newUserStatus

        if type_ == 'gtk':
            func = self.tray.set_from_pixbuf
        elif type_ == 'egg':
            func = self.image.set_from_pixbuf

        if pixbuf == None:
            func(self.theme.statusToPixbuf(newUserStatus))
        else:
            func(pixbuf)

        if self.controller.msn and self.controller.userEmail:
            text = 'Emesene - ' + str(self.controller.userEmail)
            if type_ == 'egg':
                self.tooltips.set_tip(self.eventBox, text)
            elif type_ == 'gtk':
                self.tray.set_tooltip(text)
        
    def buildTrayIconWin32(self):
        '''Build the trayIcon for windows'''

        ## http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/winui/windowsuserinterface/resources/icons/iconreference/iconfunctions/loadicon.asp
        hicon = win32gui.ExtractIcon(0, 'themes\\' + \
            self.config.user['theme'] + '\\' + 'trayicon.ico', 0)
        self.win32ext.add_notify_icon(hicon, 'Emesene') # TODO: account
        self.win32ext.notify_icon.menu = self.menu

        # Set up the callback messages
        self.win32ext.message_map({WM_TRAYMESSAGE: self.iconClickWin32})

    def iconClickWin32(self, hwnd, message, wparam, lparam):
        '''the event handler for windows'''

        if lparam == WM_RBUTTONUP:
            self.win32ext.notify_icon.menu.popup(None, None, None, 0, 0)
        elif lparam == WM_LBUTTONUP:
            self.win32ext.notify_icon.menu.popdown()
            self.showHide()

    def iconClickPosix(self, widget, event):
        '''the event handler for linux'''

        if event.type == gtk.gdk.BUTTON_PRESS: # Single click
            if event.button == 1: # Left Click
                self.showHide(self.eventBox)
            elif event.button == 3: # Right Click - Show popup
                self.menu.popup(None, None, None, event.button, event.time)
                
    def on_popup_menu( self, status_icon, button, activate_time ):
        self.menu.popup( None, None, None, button, activate_time )
        
    def on_activate( self, status_icon ):
        self.showHide()
                
    def buildMenu(self):
        '''Build the menu widget'''

        self.menu = gtk.Menu()
        
        menuItem = gtk.ImageMenuItem( gtk.STOCK_QUIT )
        menuItem.connect('activate', self.on_quit)

        self.menu.append(menuItem)
        self.menu.show_all()
    
    def on_quit( self, menuitem):
        self.controller.quit( 0 )
    
    def showHide(self, widget = None):
        '''Show or hide the main window'''

        if self.mainWindow.flags() & gtk.VISIBLE:
            self.mainWindow.hide()
        else:
            self.mainWindow.deiconify()
            self.mainWindow.show()
            
    def getNotifyObject( self ):
        if not disabled and not type_ == 'gtk':
            return self.tray
        
        return None



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# based on plugin by Tomer Haimovich

VERSION = '0.2'
IFACE_NAME = 'org.gnome.Listen'
IFACE_PATH = '/org/gnome/listen'

import CurrentSong


class Listen( CurrentSong.DbusBase ):
    '''Communicate with GNOME's Listen player'''
    
    def __init__(self):
        CurrentSong.DbusBase.__init__( self, IFACE_NAME, self.setInterface )

        try: self.iface
        except: self.iface = None

    def setInterface( self ):
        self.iface = self.bus.get_object( IFACE_NAME, IFACE_PATH )
        
    def check(self):
        if not self.iface or not self.isNameActive(IFACE_NAME):
            return
        
        message = self.iface.current_playing()
        
        # process the received string. it's built this way:
        #  <m_track> - (<m_album> - <m_artist>)
        
        if message != "":
            buf = message.split(" - (")
            title = buf[0]
            secbuf = buf[1].split(" - ")
            album = secbuf[0]
            artist = secbuf[1][0:-1]
            
            if title != self.title or \
               album != self.album or \
               artist != self.artist:
                self.title = title
                self.album = album
                self.artist = artist
                return True
            else:
                return False
            
    def isPlaying(self):
        if not self.isRunning(): return False
        if not self.iface: return False
        try:
            self.iface.current_playing()
        except:
            return False
        return True
        
    def isRunning(self):
        return self.isNameActive(IFACE_NAME)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import Plugin

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = _( 'Edit your config file' )
        self.authors = { 'j0hn' : 'j0hn.com.ar@gmail.com' }
        self.website = ''
        self.displayName = _( 'Tweaks' )
        self.name = 'Tweaks'
        
        self.config = controller.config
                
        self.controller = controller
        

    def start( self ):
        '''start the plugin'''
        self.enabled = True
    
    def stop( self ):    
        '''stop the plugin'''
        self.enabled = False
                
    def action( self ):
        pass
    
    def check( self ):
        return ( True, 'Ok' )
    
    def configure( self ):
        l = []
        configDict = {}
        
        for key, value in self.config.user:
            configDict[key] = value
        
        config = Plugin.Option('config', dict, 'Custom config', \
            'Config file', None, configDict)
        l.append( config )
        window = Plugin.ConfigWindow( _( 'Config config' ), l )
        window.set_default_size(400, -1)
        response = window.run()
        
        configWidget = config.widget
        
        if response != None:
            for key in configWidget.data.iterkeys():
            
                if configWidget.data[key] == "1":
                    configWidget.data[key] = True
                elif configWidget.data[key] == "0":
                    configWidget.data[key] = False
                    
                if self.config.user[key] != configWidget.data[key]:
                    self.config.user[key] = configWidget.data[key]
        return True




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import re
import time
import emesenelib.common
from emesenecommon import PATH
from Parser import ConversationDataType


tagRe = re.compile('%(.*?)%')

class ConversationLayoutManager(object):
    ''' This class handles the conversation theming system '''
    
    def __init__(self, controller):
        self.infos = {}
        self.theme = {}
        self.reset()
        self.setDefault()        
        self.controller = controller
        self.config = controller.config
        self.unifiedParser = controller.unifiedParser
         
    def reset(self):
        ''' Resets the theme '''
        self.infos['name'] = ''
        self.infos['description'] = ''
        self.infos['author'] = ''
        self.infos['website'] = ''
        self.theme['incoming'] = ''
        self.theme['consecutive_incoming'] = ''
        self.theme['offline_incoming'] = ''
        self.theme['outgoing'] = ''
        self.theme['consecutive_outgoing'] = ''
        self.theme['information'] = ''
        self.theme['error'] = ''
    
    def setValue(self, key, value):
        ''' Sets a key according to the given value '''
        if key in self.theme.keys():    
            self.theme[key] = value
        elif key in self.infos.keys():
            self.infos[key] = value
        else:
            emesenelib.common.debug(key + ' is not a valid property for a conversation layout !')
            
    def resolveValue(self):
        ''' Resolve shortcut like "incoming=outgoing" '''
        for key, value in self.theme.iteritems():
            if value in self.theme.keys():
                self.theme[key] = self.theme[value]
                
    def setDefault(self):
        ''' Sets the default theme (you should use that when a load fails) '''
        self.theme['incoming'] = '<span style="font-weight: bold;">%nick% says :</span><br/>    [%h%:%m%:%s%] %formattedmessage%<br/>'
        self.theme['consecutive_incoming'] = '    [%h%:%m%:%s%] %formattedmessage%<br/>'
        self.theme['offline_incoming'] = 'incoming'
        self.theme['outgoing'] = 'incoming'
        self.theme['consecutive_outgoing'] = 'consecutive_incoming'
        self.theme['information'] = '<span style="font-weight: bold;">%message%</span><br/>'
        self.theme['error'] = '<span style="font-weight: bold; color: #FF0000;">%message%</span><br/>'
        self.resolveValue()
                        
    def load(self, name):
        '''Loads a theme, return true on success, false otherwise.
        It seeks the theme in config dir, and then in app dir.
        If the loads fails, the old layout is restored''' 
        def doLoad(filename):
            self.reset()
            themefile = None
            try:
                themefile = open(filename, 'r')
                themestring = themefile.read()
                
                for i in themestring.splitlines():
                    i = i.lstrip()
                    if i != '' and not i.startswith('#'):
                        delim = i.find('=')
                        key = i[:delim].lower()
                        value = i[delim+1:]
                        self.setValue(key, value)
                
                self.resolveValue()
                themefile.close()
                
                if not self.isValid():
                    return False
                else:
                    return True
                    
            except:
                if themefile:
                    themefile.close()
                return False
        
        oldInfos = self.infos
        oldTheme = self.theme
        
        filename = PATH.CONVTHEMES_HOME_PATH + os.sep + name + os.sep + 'theme'
        if not os.path.exists(filename):
            filename = PATH.CONVTHEMES_SYSTEM_WIDE_PATH + os.sep + name + \
                os.sep + 'theme'
        
        if os.path.exists(filename):
            success = doLoad(filename)
        else:
            success = False
            
        if not success:
            self.infos = oldInfos
            self.theme = oldTheme
            
        return success
          
    def isValid(self):
        ''' Checks if the theme is valid '''
        for value in self.theme.values():
            if value == None or value in self.theme.keys():
                return False         
        return True
                
    def listAvailableThemes(self):
        ''' Lists all available and valid themes and return a list '''
        conversationLayout = ConversationLayoutManager(self.controller)
        validThemesList = []
        
        try:
            homethemes = os.listdir(PATH.CONVTHEMES_HOME_PATH)
        except OSError:
            homethemes = []
        try:
            globalthemes = os.listdir(PATH.CONVTHEMES_SYSTEM_WIDE_PATH)
        except OSError:
            globalthemes = []
        
        for currentTheme in homethemes + globalthemes:
            if conversationLayout.load(currentTheme):
                validThemesList.append(currentTheme)
                                
        return validThemesList

    def layout(self, username, message, format, conversation, type, timestamp, \
               ink=False):
        ''' Returns HTML code according to the the current theme '''
        
        if type not in self.theme.keys():
            return ''
            
        switchboard = conversation.switchboard

        if not ink:
            messageDataObject = self.unifiedParser.getParser(message, ConversationDataType)
            messageDataObject.setConversation(conversation)
            messageDataObject.setUser(username)

            if self.config.user['parseSmilies']:
                message = messageDataObject.get()
            else:
                message = messageDataObject.get(smileys=False)
            
            message = message.replace('\n', '<br/>')
        else:
            message = '<img src="file://%s" alt="Ink message" />' % message
            
        arguments = {}
        
        # Fills nick
        arguments['nick'] = ''
        if username != None:
            arguments['nick'] = self.controller.msn.getUserDisplayName(username)
                
        # Fills message
        arguments['message'] = message
        
        # Fills avatar
        arguments['avatar'] = ''
        if type == 'outgoing' or type == 'consecutive_outgoing':
            arguments['avatar'] = self.controller.avatar.getImagePath()
        elif type == 'incoming' or type == 'consecutive_incoming':
            arguments['avatar'] = self.controller.msn.cacheDir + os.sep + username.split('@')[0] + '.tmp'        
        
        # Fills (?) timestamp
        arguments['timestamp'] = timestamp

        # Layout
        if format is None:
            parser = LayoutTagParser(self, arguments, self.getCssStyle(), type)
        else:
            parser = LayoutTagParser(self, arguments, format, type)
            
        result = tagRe.sub(parser.sub, self.theme[type])
        
        return '<span><span style="font-size: %spt;">%s</span></span>' % \
               (self.config.user['fontSize'], result)
               
    def getPreview(self):
        ''' Returns a preview of the layout '''
        preview = ''
        arguments = {}
        userStyle = self.getCssStyle()
        
        # Fills arguments with faked ones
        # First message
        arguments['nick'] = self.controller.msn.nick
        arguments['message'] = _('This is an outgoing message')
        arguments['avatar'] = self.controller.avatar.getImagePath()
        arguments['timestamp'] = time.time()
        parser = LayoutTagParser(self, arguments, userStyle, 'outgoing')
        preview += tagRe.sub(parser.sub, self.theme['outgoing'])
        
        # Second message
        arguments['message'] = _('This is a consecutive outgoing message')
        parser = LayoutTagParser(self, arguments, userStyle, 'consecutive_outgoing')
        preview += tagRe.sub(parser.sub, self.theme['consecutive_outgoing'])
        
        # Third message
        arguments['nick'] = 'John Doe'
        arguments['message'] = _('This is an incoming message')
        # TODO : have an example picture
        arguments['avatar'] = self.controller.avatar.getImagePath()
        parser = LayoutTagParser(self, arguments, None, 'incoming')
        preview += tagRe.sub(parser.sub, self.theme['incoming'])
        
        # Fourth message
        arguments['message'] = _('This is a consecutive incoming message')
        parser = LayoutTagParser(self, arguments, None, 'consecutive_incoming')
        preview += tagRe.sub(parser.sub, self.theme['consecutive_incoming'])
        
        # Information message
        arguments['message'] = _('This is an information message')
        parser = LayoutTagParser(self, arguments, None, 'information')
        preview += tagRe.sub(parser.sub, self.theme['information'])
        
        # Fourth message
        arguments['message'] = _('This is an error message')
        parser = LayoutTagParser(self, arguments, None, 'error')
        preview += tagRe.sub(parser.sub, self.theme['error'])       
        
        return '<span><span style="font-size: %spt;">%s</span></span>' % \
               (self.config.user['fontSize'], preview)
               
    def getCssStyle(self):
        '''return the css style of the user'''
    
        style = ''

        if self.config.user['fontBold']:
            style += 'font-weight: bold;'

        if self.config.user['fontItalic']:
            style += 'font-style: italic;'

        if self.config.user['fontUnderline']:
            if self.config.user['fontStrike']:
                style += 'text-decoration: underline line-through;'
            else:
                style += 'text-decoration: underline;'
        elif self.config.user['fontStrike']:
            style += 'text-decoration: line-through;'

        style += 'font-size: ' + str(self.config.user['fontSize']) + 'pt;'
        style += 'color: ' + str(self.config.user['fontColor']) + ';'
        style += 'font-family: ' + str(self.config.user['fontFace']) + ';'
        
        return style
        
    def getName(self):
        return self.infos['name']
    
    def getDescription(self):
        return self.infos['description']
        
    def getAuthor(self):
        return self.infos['author']
        
    def getWebsite(self):
        return self.infos['website']
        
class LayoutTagParser:
    '''thread-safe environment to parse layout tags'''

    def __init__(self, parent, arguments, format, type):
        self.parent = parent
        self.arguments = arguments
        self.format = format
        self.type = type

        self.controller = parent.controller
        self.config = parent.controller.config
        self.unifiedParser = parent.unifiedParser

    def sub(self, data):
        ''' Called by re.sub for each tag found '''
        tag = data.groups()[0].lower()

        if tag == 'nick':
            nick = self.arguments['nick']
            
            nickDataObject = self.unifiedParser.getParser(nick, ConversationDataType)
            if self.config.user['parseSmilies']:
                nick = nickDataObject.get(urls=False)
            else:
                nick = nickDataObject.get(urls=False, smileys=False)
            
            return nick
                    
        elif tag == 'message' or \
             (self.config.user['disableFormat'] and tag == 'formattedmessage'):
            return self.arguments['message']
        
        elif tag == 'formattedmessage':
            message = self.arguments['message']
            return '<span style="%s">%s</span>' % (self.format, message)
            
        elif tag == 'h':
            return time.strftime('%H', time.localtime(self.arguments['timestamp']))
         
        elif tag == 'm':
            return time.strftime('%M', time.localtime(self.arguments['timestamp']))
         
        elif tag == 's':
            return time.strftime('%S', time.localtime(self.arguments['timestamp']))
             
        elif tag == 'date':
            return time.strftime('%x', time.localtime(self.arguments['timestamp']))

        elif tag == 'avatar':               
            return '<img src="file://' + self.arguments['avatar'] + '"/>'
        
        elif tag == 'says':
            return _('says')

        else:
            return data.groups()[0]
        
    def extractArgument(self, tag):
        ''' Retrieves arguments from a tag '''
        start = tag.find('(')+1
        end = tag.find(')')
        arglist = tag[start:end].replace(' ', '')
        return arglist.split(',')




# -*- coding: utf-8 -*-

'''module with functions to export conversation logs'''

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
#    USA


import os
import sys
import time

import emesenelib.common
import plugins_base.Logger as Logger

# limit the result to 10000 messages, it can be changed..
RESULT_LIMIT = 10000

def parse_format(format):
    '''parse the format and return the style'''

    # FN=Sans; EF=; CO=000000; PF=0
 
    style = ''
 
    if format.find("FN=") != -1:
        font = format.split('FN=')[1].split(';')[0].replace('%20', ' ')
        style += 'font-family: ' + emesenelib.common.escape(font) + ';'

    if format.find("CO=") != -1:
        color = format.split('CO=')[1].split(';')[0]
             
        if len(color) == 3:
            color = color[2] + color[1] + color[0]
            style += 'color: #' + emesenelib.common.escape(color) + ';'
        else:
            color = color.zfill(6)
            
        if len(color) == 6:
            color = color[4:6] + color[2:4] + color[:2]
            style += 'color: #' + emesenelib.common.escape(color) + ';'

    if format.find("EF=") != -1:
        effect = set(format.split('EF=')[1].split(';')[0])

        if "B" in effect: 
            style += 'font-weight: bold;'
        if "I" in effect: 
            style += 'font-style: italic;'
        if "U" in effect: 
            style += 'text-decoration: underline;'
        if "S" in effect: 
            style += 'text-decoration: line-through;'
    
    return style

def format_message_plain(date, nick, message):
    '''return a plain text message'''
    (_format, encoding, text) = message.split('\r\n')
    return '[' + date + '] ' + nick + ': ' + text

def format_message_html(date, nick, message):
    '''return a plain text message'''
    (_format, encoding, text) = message.split('\r\n')
    style = parse_format(_format)
    formated_message =  '<span style="%s">%s</span>' % (style, text)

    return '[' + date + '] <em>' + nick + '</em>: ' + formated_message + '<br/>'

def export(filename, mail, format, out):
    '''export the conversations from the user with mail "mail" located
    on the logs in "filename" using the format "format" writing to
    the file-like object out (use sys.stdout to print or StringIO to
    get a string)'''

    if format == 'html':
        format_fun = format_message_html
    else:
        format_fun = format_message_plain

    if not os.access(filename, os.R_OK):
        out.write('cant read from file %s' % (filename,))
        return False
        
    logger = Logger.Logger(filename)
    put_new_line = True

    if format == 'html':
        out.write('<html><body>')

    for id_conversation in logger.get_conversation_ids(mail):
        if put_new_line:
            if format == 'html':
                out.write('<br/>')
            else:
                out.write('\n\n')

            put_new_line = False

            if export_conversation(logger, id_conversation, out
                , RESULT_LIMIT) > 0:
                put_new_line = True 

    if format == 'html':
        out.write('</body></html>')

    return True

def export_conversation(logger, id_conversation, out, msg_limit=10000, 
    format_fun=format_message_html):
    '''export one conversation, return the number of messages printed
    (0 means that this conversation was to request a DP or something)'''
    nick_cache = {}
    msg_count = 0

    for (stamp, mail, message) in logger.get_conversation(id_conversation, 
                                                        msg_limit):

        if mail in nick_cache:
            if nick_cache[mail]['next'] is not None and \
                nick_cache[mail]['next'] <= stamp:
                nick = logger.get_user_nick(mail, stamp)
                next = logger.get_next_nick_stamp(mail, stamp) 
                nick_cache[mail] = {'nick': nick, 'next' : next}
            else:
                nick = nick_cache[mail]['nick']
        else:
            nick = logger.get_user_nick(mail, stamp)
            next = logger.get_next_nick_stamp(mail, stamp) 
            nick_cache[mail] = {'nick': nick, 'next' : next}
            
        date = time.ctime(float(stamp))
        formated_message = format_fun(date, nick, message)
        out.write(formated_message)
        out.write('\n')
        msg_count += 1

    return msg_count

if __name__ == '__main__':
    if len(sys.argv) != 4:
        print 'USAGE: %s /path/to/log_file.db contact_mail format' % \
            (sys.argv[0],)
        print 'format: [html|txt]'
    
    export(sys.argv[1], sys.argv[2], sys.argv[3], sys.stdout)



"""
Simple desktop integration for Python. This module provides desktop environment
detection and resource opening support for a selection of common and
standardised desktop environments.

Copyright (C) 2005, 2006 Paul Boddie <paul@boddie.org.uk>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

--------

Desktop Detection
-----------------

To detect a specific desktop environment, use the get_desktop function.
To detect whether the desktop environment is standardised (according to the
proposed DESKTOP_LAUNCH standard), use the is_standard function.

Opening URLs
------------

To open a URL in the current desktop environment, relying on the automatic
detection of that environment, use the desktop.open function as follows:

desktop.open("http://www.python.org")

To override the detected desktop, specify the desktop parameter to the open
function as follows:

desktop.open("http://www.python.org", "KDE") # Insists on KDE
desktop.open("http://www.python.org", "GNOME") # Insists on GNOME

Without overriding using the desktop parameter, the open function will attempt
to use the "standard" desktop opening mechanism which is controlled by the
DESKTOP_LAUNCH environment variable as described below.

The DESKTOP_LAUNCH Environment Variable
---------------------------------------

The DESKTOP_LAUNCH environment variable must be shell-quoted where appropriate,
as shown in some of the following examples:

DESKTOP_LAUNCH="kdialog --msgbox"       Should present any opened URLs in
                                        their entirety in a KDE message box.
                                        (Command "kdialog" plus parameter.)
DESKTOP_LAUNCH="my\ opener"             Should run the "my opener" program to
                                        open URLs.
                                        (Command "my opener", no parameters.)
DESKTOP_LAUNCH="my\ opener --url"       Should run the "my opener" program to
                                        open URLs.
                                        (Command "my opener" plus parameter.)

Details of the DESKTOP_LAUNCH environment variable convention can be found here:
http://lists.freedesktop.org/archives/xdg/2004-August/004489.html
"""

__version__ = "0.2.3"

import os
import sys

try:
    import subprocess
    def _run(cmd, shell, wait):
        opener = subprocess.Popen(cmd, shell=shell)
        if wait: opener.wait()
        return opener.pid

except ImportError:
    import popen2
    def _run(cmd, shell, wait):
        opener = popen2.Popen3(cmd)
        if wait: opener.wait()
        return opener.pid

import commands
import webbrowser  # fallback

# global
override = ''

def get_desktop(dontoverride=False):

    """
    Detect the current desktop environment, returning the name of the
    environment. If no environment could be detected, None is returned.
    """

    global override
    if override and not dontoverride:
        return 'override'
    elif os.environ.has_key("KDE_FULL_SESSION") or \
        os.environ.has_key("KDE_MULTIHEAD"):
        return "KDE"
    elif os.environ.has_key("DESKTOP_SESSION") and \
        os.environ['DESKTOP_SESSION'] == 'xfce4':
        return 'xfce4'
    elif os.environ.has_key("GNOME_DESKTOP_SESSION_ID") or \
        os.environ.has_key("GNOME_KEYRING_SOCKET"):
        return "GNOME"
    elif sys.platform == "darwin":
        return "Mac OS X"
    elif hasattr(os, "startfile"):
        return "Windows"
    else:
        return None

def is_standard():

    """
    Return whether the current desktop supports standardised application
    launching.
    """

    return os.environ.has_key("DESKTOP_LAUNCH")

def open(url, desktop=None, wait=0):

    """
    Open the 'url' in the current desktop's preferred file browser. If the
    optional 'desktop' parameter is specified then attempt to use that
    particular desktop environment's mechanisms to open the 'url' instead of
    guessing or detecting which environment is being used.

    Suggested values for 'desktop' are "standard", "KDE", "GNOME", "Mac OS X",
    "Windows" where "standard" employs a DESKTOP_LAUNCH environment variable to
    open the specified 'url'. DESKTOP_LAUNCH should be a command, possibly
    followed by arguments, and must have any special characters shell-escaped. 

    The process identifier of the "opener" (ie. viewer, editor, browser or
    program) associated with the 'url' is returned by this function. If the
    process identifier cannot be determined, None is returned.

    An optional 'wait' parameter is also available for advanced usage and, if
    'wait' is set to a true value, this function will wait for the launching
    mechanism to complete before returning (as opposed to immediately returning
    as is the default behaviour).
    """

    # Attempt to detect a desktop environment.

    detected = get_desktop()
    
    # Give higher priority to user preference
    
    if (desktop is None or desktop == "override") and detected == "override":
        global override
        arg = override.replace("%url%", commands.mkarg(url))
        return _run(arg, 1, wait)

    # Desktops whose existence can be easily tested.

    elif (desktop is None or desktop == "standard") and is_standard():
        arg = "".join([os.environ["DESKTOP_LAUNCH"], commands.mkarg(url)])
        return _run(arg, 1, wait)

    elif (desktop is None or desktop == "Windows") and detected == "Windows":
        # NOTE: This returns None in current implementations.
        try:
            return os.startfile(url)
        except OSError:
            # some broken configs may return error here but work anyway
            return
    elif desktop is None:
        desktop = detected
            
    cmd = get_command(desktop, url)
    if cmd:
        return _run(cmd, 0, wait)
    else:
        # Finish with an error where no suitable desktop was identified.
        webbrowser.open(url)


def get_command(desktop, url):
    '''Test for desktops where the overriding is not verified.'''
    if desktop == "KDE":
        return ["kfmclient", "exec", url]
    elif desktop == "GNOME":
        return ["gnome-open", url]
    elif desktop == 'xfce4':
        return ["exo-open", url]
    elif desktop == "Mac OS X":
        return ["open", url]
    elif desktop == "standard":
        return ['$DESKTOP_LAUNCH']
    elif desktop == "Windows":
        return ['os.startfile()']
    else:
        return None

# vim: tabstop=4 expandtab shiftwidth=4



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from uuid import uuid4
from string import Template
import XmlTemplates
import SignalHandler
import SoapManager
from xml.dom.minidom import parseString
import email
import time
import common
from xml.parsers.expat import ExpatError

class SoapHandle( object ):
    def __init__( self, msnOIM, template, d ):
        self.template = template
        self.d = d
        self._passportid = msnOIM._passportid
        self.msnOIM = msnOIM
        self.msn = self.msnOIM.msn
        self.retrys = 0
        
    def handle(self, response):
        '''Handle the Soap Request'''
        to, message = self._parseSentMessage()

        try:
            xml = parseString(response.body)
        except ExpatError, e:
            self.msn.emit('send-message-error', to, message, str(e))
            print to, message, str(e)
            return

        # TweenerChanllenge:
        if( xml.getElementsByTagName('TweenerChallenge').length>0 ):
            try:
                self.tweenerChallenge(xml)
            except Exception, e:
                self.msn.emit( 'send-message-error', to, message, str(e))
            return self.retry(response)

        # LockKeyChallenge:
        if(xml.getElementsByTagName( 'LockKeyChallenge').length>0 ):
            self.lockKeyChallenge( xml )
            return self.retry(response)
        
        if xml.getElementsByTagName('GetMessageResult').length > 0:
            self.msn.emit('offline-message-received',
                self.msnOIM.parseOIM(response))

        fault = ''
        if xml.getElementsByTagName('faultcode').length > 0:
            fault = xml.getElementsByTagName('faultcode')[0]\
                .firstChild.nodeValue
            if fault.strip() != '':
                #self.msn.emit('send-message-error', to, message, fault)
                print "OIM: line 69:",
                print to, message, fault
                return

    def _parseSentMessage(self):
        try:
            to = self.d['to']
            message = self.d['content'].decode('base64')
            return (to, message)
        except:
            return ('', '')

    def retry(self, response):
        '''Retry to send Soap Request with new data'''
        if self.retrys < 3:
            self.retrys += 1
            body = self.template.safe_substitute( self.d )
            
            newRequest = SoapManager.SoapRequest( response.action,
                response.host, response.port, response.path, body,
                response.callback )
            SoapManager.put( newRequest )
        elif self.retrys > 3:
            to, message = self._parseSentMessage()
            self.msn.emit( 'send-message-error', to, message, 'Auth Error' )
            
    def tweenerChallenge( self, xml ):
        '''Retreive new PassportId'''
        # we get the hash
        hash = xml.getElementsByTagName('TweenerChallenge')[0].firstChild.nodeValue
        # start the passport authentication
        self.msn.passportReAuth(hash)
        self.msnOIM._passportid =  self._passportid = self.msn.passportid

        self.d['passport'] = self._passportid.replace('&','&amp;')
        s=self._passportid[2:]

        t,p = s.split('&p=')
        self.msnOIM._t = t
        self.msnOIM._p = p
        #Reset d
        self.d['t'], self.d['p'] = t, p
            
    def lockKeyChallenge( self, xml ):
        '''Retreive new lockKey'''
        hash = xml.getElementsByTagName('LockKeyChallenge')[0].firstChild.nodeValue
        lockKey = SignalHandler.doChallenge(str(hash))
        self.msnOIM._lockKey = lockKey
        #Reset d
        self.msnOIM._lockKey = self.d['lockKey'] = lockKey
        

class MsnOIM(object):
    '''This is the class to handle msn oim'''

    def __init__(self,msn):
        self.msn=msn
        self._lockKey=''
        self._passportid=''
        self._seq=1
        self._t=''
        self._p=''

        self.msg_list = []

    def destroy( self ):
        self.msn = None
        self.msg_list = []

    def getMessageCount( self ):
        '''Return the number of messages waiting'''
        
        return len( self.msg_list )
    
    def getMessages( self ):
        '''Return the msg_lst'''
        
        return self.msg_list

    def send(self,to,msg):
        '''Send an offline message to friend'''
        common.debug('OIM To: ' + str(to), 'msnoim')
        soapTemplate=Template(XmlTemplates.send_message)
        #From: =?utf-8?B?OjpSb2dlcjo6?= &lt;rogerduran@gmail.com&gt;
        nick='=?%s?%s?=%s?=' % ('utf-8','B',self.msn.nick.encode('base64').strip())

        d=dict(memberName=self.msn.user,friendlyName=nick,ver='MSNP13',buildVer='8.0.0792',to=to,passport=self._passportid.replace('&','&amp;'),appid=SignalHandler._PRODUCT_ID,lockKey=self._lockKey,seqNum=self._seq,runId=uuid4().__str__(),content=msg.encode('base64').strip())
        self._seq=self._seq+1

        self.soapRequest('http://messenger.msn.com/ws/2004/09/oim/Store','Send OIM','ows.messenger.msn.com',443,'/OimWS/oim.asmx',soapTemplate, d)

    def parseMailData(self,mailData):
        ''' Parse the mail data xml'''
        if mailData == 'too-large':
            return self.tooLarge()
        mailDataDom=parseString(mailData)
        for m in mailDataDom.getElementsByTagName('M'):
            item={}
            for node in m.childNodes:
                if node.nodeType!=node.TEXT_NODE and node.childNodes.length==1:
                    item[node.nodeName]=node.firstChild.nodeValue
            self.msg_list.append(item)        
        return False
        
    def parseOIM( self, response ):
        data = response.body.split('<GetMessageResult>')[1].split('</GetMessageResult>')[0]
        oim = email.message_from_string( data )
        
        user = self.parseFrom( oim.get('From') )
        message = oim.get_payload().decode('base64')
        
        date = email.Utils.parsedate_tz( oim.get( 'Date' ) )
        date = time.localtime( email.Utils.mktime_tz( date ) )
        
        return [user, date, message]

    def tooLarge( self ):
        '''Return the mail data using soap if there are too many OIMs'''
        t,p=self.msn.passportid[2:].split('&p=')
        request = SoapManager.SoapRequest( 'http://www.hotmail.msn.com/ws/2004/09/oim/rsi/GetMetadata',\
                                'rsi.hotmail.com', 443,\
                                '/rsi/rsi.asmx', \
                                XmlTemplates.getMailData%(t,p) )
        SoapManager.put( request )
        response = SoapManager.get()
        return self.parseMailData( response.body )

    def retrieve(self,message_id):
        '''retreive an oim message by message_id'''
        if(self.msn.passportid!=''):
            s=self.msn.passportid[2:]
            self._t,self._p=s.split('&p=')

        soapTemplate=Template(XmlTemplates.retreive_message)
        d=dict(t=self._t,p=self._p,mid=message_id)

        self.soapRequest('http://www.hotmail.msn.com/ws/2004/09/oim/rsi/GetMessage','Retreive OIM','rsi.hotmail.com',443,'/rsi/rsi.asmx',soapTemplate,d)
        self.delete( message_id )

    def getMessage( self, xmlString ):
        '''Return the message from the xml'''
        
        dom=parseString(xmlString)
        return dom.getElementsByTagName('GetMessageResult')[0].firstChild.nodeValue

    def getSender( self, xml ):
        '''return the From of the message'''

        # From: =?utf-8?B?V2FyIFsgd3d3LmlnbmFjaW9ndWVycmEuY29tLmFyIF0gWyBDZWx1bGFyIG51ZXZv?= &lt;iogua@gmail.com&gt;
        try:
            return xml.split( 'From: =?' )[ 1 ].split( '&lt;' )[ 1 ].split( '&gt;' )[ 0 ]
        except:
            return '?'

    def parseFrom( self, mail ):
        '''Parse encoded from'''

        data = mail.split('?')
        # oooohh shi-
        if len(data) < 3:
            return {'name': '', 'addr': ''}
        
        if data[2] == 'B':
            name = data[3].decode('base64').decode( data[1] )
            addr = data[4][6:][:-4]
        elif data[2] == 'Q':
            #Quoted Messsage.. TODO
            name = addr = 'TODO'
        return { 'name':name, 'addr':addr }

    def delete(self,message_id):
        '''Delete an oim message'''
        if(self.msn.passportid!=''):
            s=self.msn.passportid[2:]
            self._t,self._p=s.split('&p=')

        soapTemplate=Template(XmlTemplates.delete_message)
        d=dict(t=self._t,p=self._p,mid=message_id)

        self.soapRequest('http://www.hotmail.msn.com/ws/2004/09/oim/rsi/DeleteMessages','Delete OIM','rsi.hotmail.com',443,'/rsi/rsi.asmx',soapTemplate,d)

    def soapRequest(self,url,description,server,port,path,soapTemplate,d):
        '''Soap request method handle Tweener challenge and Lock key challenge'''
        soapHandle = SoapHandle( self, soapTemplate, d )
        body=soapTemplate.safe_substitute(d)
        request = SoapManager.SoapRequest( url, server, port, path, body, soapHandle.handle )
        SoapManager.put( request )



# -*- coding: utf-8 -*-
'''a module to handle groups'''

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import abstract.GroupManager

class GroupManager(abstract.GroupManager.GroupManager):
    '''implementation of asbtract.GroupManager.GroupManager
    using emesenelib'''

    def __init__(self, dialog, protocol):
        '''initialize the object, dialog is a implementation
        of abstract.dialog, it's used to interact with the user'''
        abstract.GroupManager.GroupManager.__init__(self, 
            dialog, protocol)

        if protocol:
            self._connect_signals()
        else:
            debug('Protocol is None')

    def _connect_signals(self):
        '''connect the protocol signals'''
        # connect the signals to the private methods, and call
        # the methods on the abstract class with the correct
        # attributes (see the __ on the methods)
        self.protocol.connect('group-added', self.__on_group_added)
        self.protocol.connect('group-removed', self.__on_group_removed)
        self.protocol.connect('group-renamed', self.__on_group_renamed)

        self.protocol.connect('contact-added-to-group', 
            self.__on_contact_added_to_group)
        self.protocol.connect('contact-removed-from-group', 
            self.__on_contact_removed_from_group)

    def __on_group_added(self, protocol, name, identifier):
        '''method called when a new group is created'''
        self._on_group_added(name, identifier)

    def __on_group_removed(self, protocol, name):
        '''method called when a group is removed'''
        self._on_group_removed(name)

    def __on_group_renamed(self, protocol, old_name, new_name):
        '''method called when the name of the group is changed'''
        self._on_group_renamed(old_name, new_name)

    def __on_contact_added_to_group(self, protocol, account, group_name):
        '''callback called when an account is added to a group'''
        self._on_contact_added_to_group(account, group_name)
            
    def __on_contact_removed_from_group(self, protocol, account, group_name):
        '''callback called when an account is removed from a group'''
        self._on_contact_removed_from_group(account, group_name)

    # public API

    def add(self, name):
        '''add a group'''
        if name:
            if self.protocol:
                self.protocol.addGroup(name)
        else:
            debug("invalid group name '%s'" % (name,))

    def rename(self, name, new_name):
        '''rename a group'''
        if new_name and name != new_name:
            if self.protocol:
                self.protocol.renameGroup(name, new_name)
        else:
            debug("invalid group name '%s'" % (new_name,))

    def remove(self, name):
        '''remove a group'''
        if self.protocol:
            self.protocol.removeGroup(name)

def debug(msg):
    '''debug method, the module send the debug here, it can be changed
    to use another debugging method'''
    return
    print('ContactManager.py: ', msg)



#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# Right now we are not using gtk themes facilities
#
# We should consider using them at some point

import re
import os
import gtk.gdk
from shutil import copyfile

from emesenecommon import PATH
import emesenelib.common

ALTERNATIVE_SMILEYS = {\
    '<:o)':['<:O)'],
    '(a)' : [ '(A)' ],\
    ':@' : [ ':-@' ],\
    '(b)' : [ '(B)' ] ,\
    '(z)' : [ '(Z)' ] ,\
    '(u)' : [ '(U)' ] ,\
    '(p)' : [ '(P)' ] ,\
    '(o)' : [ '(O)' ] ,\
    '(c)' : [ '(C)' ] ,\
    ':s' : [ ':S', ':-S' ] ,\
    ':|' : [ ':-|' ] ,\
    '(d)' : [ '(D)' ] ,\
    '(e)' : [ '(E)' ] ,\
    ':$' : [ ':-$' ] ,\
    '(g)' : [ '(G)' ] ,\
    '(x)' : [ '(X)' ] ,\
    '(h)' : [ '(H)' ] ,\
    '(i)' : [ '(I)' ] ,\
    '(m)' : [ '(M)' ] ,\
    ':D' : [ ':d', ':-d', ':-D', ':>' ] ,\
    '(l)' : [ '(L)' ] ,\
    '(k)' : [ '(K)' ] ,\
    '(f)' : [ '(F)' ] ,\
    ':(' : [ ':-(', ':<' ] ,\
    ':)' : [ ':-)' ] ,\
    ':p' : [ ':P', ':-P' ] ,\
    ':-O' : [ ':O', ':o', ':-o' ] ,\
    '(t)' : [ '(T)' ] ,\
    '(n)' : [ '(N)' ] ,\
    '(y)' : [ '(Y)' ] ,\
    '(w)' : [ '(W)' ] ,\
    ';)' : [ ';-)' ] ,\
    '(r)' : [ '(R)' ],\
}

SMILEY_TO_NAME = {\
    '<:o)' : 'party_smiley',\
    '(ap)' : 'Airplane',\
    '(a)' : 'angel',\
    ':@' : 'angry',\
    '(au)' : 'Auto',\
    '8o|' : 'Baring_teeth_smiley',\
    '(b)' : 'beer',\
    '(^)' : 'Birthday_cake',\
    '(bah)' : 'Black_sheep',\
    '(nah)' : 'goat',\
    '(||)' : 'Bowl',\
    '(z)' : 'Boy',\
    '(u)' : 'brheart',\
    '(p)' : 'photo',\
    '(@)' : 'pussy',\
    '(ci)' : 'Cigarette',\
    '(o)' : 'Clock',\
    '(c)' : 'coffee',\
    '(co)' : 'computer',\
    ':s' : 'confused',\
    ':\'(' : 'cry',\
    '(6)' : 'devil',\
    ':|' : 'stare',\
    '(&)' : 'Dog_face',\
    ':-#' : 'Dont_tell_anyone',\
    '(d)' : 'drink',\
    '(e)' : 'mail',\
    ':$' : 'blushing',\
    '8-)' : 'Eye_rolling_smiley',\
    '(~)' : 'video-x-generic',\
    '(yn)' : 'Fingerscrossed',\
    '(g)' : 'Gift_with_a_bow',\
    '(x)' : 'Girl',\
    '(%)' : 'cuffs',\
    '(h5)' : 'Hi_five',\
    '(h)' : 'coolglasses',\
    ':^)' : 'andy',\
    '(ip)' : 'Island_with_a_palm_tree',\
    '({)' : 'hugleft',\
    '(i)' : 'lamp',\
    '(li)' : 'weather-storm',\
    '(m)' : 'Messenger',\
    '(mp)' : 'phone',\
    '(mo)' : 'Money',\
    '8-|' : 'face-glasses',\
    '(8)' : 'audio-x-generic',\
    ':D' : 'grin',\
    '(pi)' : 'Pizza',\
    '(pl)' : 'Plate',\
    '(r)' : 'rainbow',\
    '(st)' : 'weather-showers-scattered',\
    '(l)' : 'emblem-favorite',\
    '(k)' : 'kiss',\
    '(f)' : 'flower',\
    '(})' : 'hugright',\
    ':(' : 'unhappy',\
    '^o)' : 'Sarcastic_smiley',\
    ':-*' : 'Secret_telling_smiley',\
    '+o(' : 'sick',\
    '(S)' : 'weather-clear-night',\
    '|-)' : 'sleeping',\
    ':)' : 'smile',\
    ':p' : 'tongue',\
    '(sn)' : 'Snail',\
    '(so)' : 'Soccer_ball',\
    '(*)' : 'star',\
    '(#)' : 'weather-clear',\
    ':-O' : 'oh',\
    '(t)' : 'Telephone_receiver',\
    '*-)' : 'Thinking_smiley',\
    '(n)' : 'thumbdown',\
    '(y)' : 'thumbup',\
    '(tu)' : 'Turtle',\
    '(um)' : 'Umbrella',\
    ':[' : 'bat',\
    ':-[' : 'bat',\
    '(w)' : 'brflower',\
    ';)' : 'wink',\
    '(xx)' : 'Xbox',\
}

ACCEPTED_FORMATS = [ 'png', 'gif', 'svg', 'xpm' ]

def resizePixbuf( pixbuf, height, width ):
    pWidth, pHeight = pixbuf.get_width(), pixbuf.get_height()
    
    if pWidth == width and pHeight == height:
        return pixbuf
            
    destX = destY = 0
    destWidth, destHeight = width, height
    if pWidth > pHeight:
        scale = float( width ) / pWidth
        destHeight = int( scale * pHeight )
        destY = int( ( height - scale * pHeight ) / 2 )
    elif pHeight > pWidth:
        scale = float( height ) / pHeight
        destWidth = int( scale * pWidth )
        destX = int( ( width - scale * pWidth ) / 2 )
    else:
        return pixbuf.scale_simple( width, height, gtk.gdk.INTERP_BILINEAR )
    
    if scale == 1:
        return pixbuf
    
    scaled = gtk.gdk.Pixbuf( gtk.gdk.COLORSPACE_RGB, True, 8, width, height)
    pixbuf.scale( scaled, 0, 0, width, height, \
                    destX, destY, scale, scale, \
                    gtk.gdk.INTERP_BILINEAR )
    return scaled

def sortedKeysByLen(adict, reverse=True):
    keys = adict.keys()
    keys.sort( lambda left, right: cmp( len(left),len(right) ) , reverse=reverse)
    return keys

class BaseTheme:
    '''Common functions to both theme classes'''

    def setTheme( self, name, small = False ):
        '''Sets the current theme to the name given.
        In case of error, it will directly fallback to the default theme
        Returns True if the specified theme was correctly set, otherwise
        False'''
        
        if name == 'default' and not small:
            self.pixbufs = self.defaults
            self.path = self.defaultPath
            return True
        else:
            self.pixbufs = {}
        
        path = None
        smallFound = False
        SMALL_PATH = self.HOME_PATH + os.sep + '.' + name + 'Small'
        if small and os.path.isdir( SMALL_PATH ):
            path = SMALL_PATH
            smallFound = True
        elif os.path.isdir( self.HOME_PATH + os.sep + name ):
            path = self.HOME_PATH + os.sep + name
        elif os.path.isdir( self.SYSTEM_WIDE_PATH + os.sep + name ):
            path = self.SYSTEM_WIDE_PATH + os.sep + name
        else:
            emesenelib.common.debug( 'Theme '+ name + \
                      ' couldn\'t be found where it was supposed to be' )
            emesenelib.common.debug( 'Falling back to the default theme' )
            self.path = self.defaultPath
            return False
        
        if small and not smallFound:
            if not os.path.isdir( SMALL_PATH ):
                path = self.makeSmall( path, SMALL_PATH )
            else:
                path = path + 'Small'
        self.small = small
        self.path = path + os.sep
        return True
        
    def makeSmall( self, path, spath ):
        return path

    def searchPixbuf( self, path, name, isLocation = False, pixbufCreator = None ):
        '''Finds name in path with ACCEPTED_FORMATS extensions
        Returns location if found and if isLocation
        Else, return pixbufCreator(location)'''
        
        for extension in ACCEPTED_FORMATS:
            location = path + name + '.' + extension
            if isLocation:
                if os.path.exists( location ):
                    return location
            else:
                try:
                    pixbuf = pixbufCreator( location )
                    return pixbuf
                except:
                    pass
        return None
        
    def getImage( self, name, animated=False ):
        '''Returns a pixbuf of the requested image from the current theme
        If the requested image name is not found in the current theme, the
        default theme's corresponding image will be returned.
        If the default theme doesn't have such a image, None is returned'''
        
        # checks if already loaded
        if name in self.pixbufs: return self.pixbufs[ name ]
        
        # determines which pixbuf function is used to create it
        if animated:
            creator = gtk.gdk.PixbufAnimation
        else:
            creator = gtk.gdk.pixbuf_new_from_file
                
        pixbuf = self.searchPixbuf( self.path, name, pixbufCreator = creator )

        if pixbuf:
            # returns icon from theme
            self.pixbufs[ name ] = pixbuf
            return pixbuf

        #else, return icon from default theme
        if name in self.defaults:
            return self.defaults[ name ]

        self.defaults[ name ] = self.searchPixbuf( self.defaultPath, name, pixbufCreator = creator)
        return self.defaults[ name ]
                
    def getImageIcon( self, name, animated=False ):
        '''Returns the location of the requested image from the current theme
        If the requested image name is not found in the current theme, the
        default theme's corresponding image will be returned.
        If the default theme doesn't have such a image, None is returned'''

        location = self.searchPixbuf( self.path, name, isLocation = True )
        if location:
            # returns icon from theme
            return location
        else:
            # will return icon from default theme
            return self.searchPixbuf( self.defaultPath, name, isLocation = True )

class Theme(BaseTheme):
    '''This class contains all the data related to a theme:
    images, icons, animations, path to sounds etc'''
    
    def __init__( self , config, name=None, sname='default' ):
        '''Constructor'''
        
        self.config = config
        self.defaults = {}
        self.pixbufs = {}
        self.HOME_PATH = PATH.THEME_HOME_PATH 
        self.SYSTEM_WIDE_PATH = PATH.THEME_SYSTEM_WIDE_PATH
        self.defaultPath = PATH.DEFAULT_THEME_PATH
        if name:
            self.setTheme( name )
        else:
            self.path = self.defaultPath
        
        self.smilies = SmilieTheme( self, sname )
        self.getSmileysList = self.smilies.getSmileysList
        self.getSmiley = self.smilies.getSmiley
        self.getSmileyPath = self.smilies.getSmileyPath
        self.smileyParser = self.smilies.smileyParser
        self.getSmileysList = self.smilies.getSmileysList
        self.getSingleSmileysList = self.smilies.getSingleSmileysList
    
    def statusToPixbuf( self, status ):
        '''Translates a status code in a pixbuf returning the according
        theme image'''
        
        name = None
        if status == 'NLN' or status == 'online':
            name = 'online'
        elif status == 'AWY' or status == 'away':
            name = 'away'
        elif status == 'BSY' or status == 'busy':
            name = 'busy'
        elif status == 'BRB' or status == 'brb':
            name = 'brb'
        elif status == 'PHN' or status == 'phone':
            name = 'phone'
        elif status == 'LUN' or status == 'lunch':
            name = 'lunch'
        elif status == 'HDN' or status == 'invisible':
            name = 'invisible'
        elif status == 'IDL' or status == 'idle':
            name = 'idle'
        elif status == 'FLN' or status == 'offline':
            name = 'offline'
        # the following status are dummys
        elif status == 'DCD' or status == 'disconnected':
            name = 'trayicon'
        elif status == 'LOG' or status == 'login':
            name = 'trayicon2'

        if name:
            return self.getImage( name )
        else:
            return None
       
    def hasUserDisplayPicture( self, contact ):
        '''return True if the user has a cached Display picture'''
        imagePath = os.path.join(self.config.getCachePath(), \
            contact.displayPicturePath)
        
        if os.path.exists( imagePath ) and os.path.isfile( imagePath ):
            return True
        else:
            return False
            
    def getUserDisplayPicture( self, contact, width=96, height=96 ):
        '''return a pixbuf with the display picture of the user
        or the default image if not found'''
        
        imagePath = os.path.join(self.config.getCachePath(), \
            contact.displayPicturePath)
        
        if os.path.exists( imagePath ) and os.path.isfile( imagePath ):
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file( imagePath )
            except:
                pixbuf = self.getImage( 'login' )
        else:
            pixbuf = self.getImage( 'login' )
        
        if width == 96 and height == 96:
            return pixbuf
        else:
            return resizePixbuf(pixbuf, width, height)

    def makeSmall( self, path, spath):
        os.mkdir( spath )
        try:
            for root, dirs, files in os.walk( path ):
                del dirs
                if root != path: continue
                for file in files:
                    name = file.split('.')[0]
                    fullname = os.path.join(root, file)
                    destname = os.path.join(spath, file)
                    if name in ('away','brb','busy','icon','idle','lunch', \
                                'invisible','mobile','offline','online', \
                                'phone'):
                        pixbuf = resizePixbuf( gtk.gdk.pixbuf_new_from_file(fullname), 16, 16 )
                        pixbuf.save(destname, 'png')
                    elif file.split('.')[1] in ACCEPTED_FORMATS:
                        copyfile(fullname,destname)
            return spath
        except Exception, e:
            print "Error creating small theme", e
            return path
    
class SmilieTheme(BaseTheme):

    def __init__( self , parent, name ):
        '''Constructor'''
        
        self.theme = parent
        self.defaults = {}
        self.pixbufs = {}
        self.defaultPath = PATH.DEFAULT_SMILIES_PATH
        self.HOME_PATH = PATH.SMILIES_HOME_PATH
        self.SYSTEM_WIDE_PATH = PATH.SMILIES_SYSTEM_WIDE_PATH
        if name:
            self.setTheme( name )
        else:
            self.path = self.defaultPath

        # sets up a dict with the different smiley shortcodes
        # that translates to smileys names
        self.smileyToName = {}
        for (key,value) in SMILEY_TO_NAME.iteritems():
            self.smileyToName[ key ] = value
            
            if key in ALTERNATIVE_SMILEYS:
                for i in ALTERNATIVE_SMILEYS[ key ]:
                    self.smileyToName[ i ] = value
        smileyPattern = ''
        
        for smiley in sortedKeysByLen( self.smileyToName ):
            smileyPattern += re.escape( emesenelib.common.escape(smiley) ) + '|'
        self.reSmiley = re.compile( '('+smileyPattern[:-1]+')' )
    
    def smileyParser( self, text, callback=None ):
        '''return text with smileys parsed in html
        you can set a callback to handle smileys parsing'''
        
        if callback == None:
            return self.reSmiley.sub( lambda data: '<img src="file://%s"/>'%self.getSmileyPath(emesenelib.common.unescape( data.groups()[0]) ) , text )
        else:
            return self.reSmiley.sub( lambda data: callback( self.getSmileyPath(emesenelib.common.unescape( data.groups()[0]) )) , text )
    

    def getSmiley( self, shortcode, animated=True ):
        '''Returns a pixbuf representing the corresponding smiley if we have one,
        otherwise None'''
            
        if shortcode in self.smileyToName:
            pixbuf = self.getImage( self.smileyToName[ shortcode ], animated )
            if not animated and type(pixbuf) == gtk.gdk.PixbufAnimation:
                pixbuf = pixbuf.get_static_image()
            return pixbuf
         
    def getSmileyPath( self, shortcode ):
        '''Returns the path of the smiley if we have one,
        otherwise None'''
        
        if shortcode in self.smileyToName:
            return self.getImageIcon( self.smileyToName[ shortcode ], True )
        
    def getSmileysList( self ):
        '''Returns a list of all the currently accepted smileys shortcodes'''
        
        return sortedKeysByLen( self.smileyToName )
    
    def getSingleSmileysList( self ):
        '''Returns a list of the currently accepted smileys shortcodes,
        without repetitions'''
        
        return sortedKeysByLen( SMILEY_TO_NAME )




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
try: import sexy
except: hasSexy = False
else: hasSexy = True

class FilterEntry(gtk.HBox):
    
    def __init__(self, callback):
        '''the callback is a function that receive
        the string typed here as only parameter'''
        
        gtk.HBox.__init__(self)
        self.set_border_width(2)
        #self.set_spacing(4)
        self.callback = callback
        
        if hasSexy:
            self.entry = sexy.IconEntry()
            self.entry.set_icon(sexy.ICON_ENTRY_PRIMARY,gtk.image_new_from_stock(gtk.STOCK_FIND,gtk.ICON_SIZE_BUTTON))
            self.entry.add_clear_button()
        else:            
            self.entry = gtk.Entry()
            iconInfo = gtk.icon_theme_get_default().lookup_icon(gtk.STOCK_FIND, 22, 0)

            self.icon = None

            if iconInfo != None:
                self.icon = gtk.Image()
                self.icon.set_from_pixbuf(iconInfo.load_icon())
                self.pack_start(self.icon, False, False)
        self.entry.connect('changed', self.entryChanged)
        self.entry.connect('key_press_event', self.entryKeypressEvent) 

        self.pack_start(self.entry)
        self.show_all()
            
    def entryChanged(self, *args):
        self.callback(self.entry.get_text())
        
    def entryKeypressEvent(self, widget, event): 
        keyval = gtk.gdk.keyval_name(event.keyval) 
        if keyval == 'Escape': 
            self.entry.props.text = '' 
            return True 
        return False 




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import time
from gobject import timeout_add, source_remove

import status

DELAY = 500
GROUP_TOOLTIP = False

class TreeViewTooltips( gtk.Window ):
    '''Class that implements the tooltips shown in the user list'''

    def __init__( self, theme, view, info_col, mail_col, type_col=-1 ):

        gtk.Window.__init__( self, gtk.WINDOW_POPUP )

        self.view = view
        self.theme = theme
        self.info_col = info_col
        self.mail_col = mail_col
        self.type_col = type_col

        self.set_name('gtk-tooltips')
        self.set_position(gtk.WIN_POS_MOUSE)
        self.set_resizable(False)
        self.set_border_width(4)
        self.set_app_paintable(True)
        self.connect('expose-event', self.on_expose_event )

        self.contactLabel = gtk.Label('')
        self.contactLabel.set_line_wrap(True)
        self.contactLabel.set_alignment(0.5, 0.5)
        self.contactLabel.show()

        self.label = gtk.Label('')
        self.label.set_line_wrap(True)
        self.label.set_alignment(0, 0.5)
        self.label.set_use_markup(True)
        self.label.show()

        self.image = gtk.Image()
        self.dataString = '<span size="small">(%s)\n\n'
        self.dataString += _( 'Blocked: %s' ) + '\n'
        self.dataString += _( 'Has you: %s' ) + '\n'
        self.dataString += _( 'Has MSN Space: %s' ) + '\n'
        # this line will be replaced with %status% since %date% \n
        # only if the logger plugin is available and dont return a
        # empty result
        self.dataString += _( '%s' ) 
        self.dataString += '</span>'
        
        self.yesNo = { 'True' : _( 'Yes' ), 'False' : _( 'No' ) }
        
        hbox = gtk.HBox( spacing=6 )
        vbox = gtk.VBox()
        hbox.pack_start( self.image )
        hbox.pack_start( vbox )

        vbox.pack_start( self.label )

        hbox.show_all()

        self.add( hbox )

        self.connect( 'delete-event', self.reset )

        view.connect('motion-notify-event', self.on_motion )
        view.connect('leave-notify-event', self.on_leave )

        self.tag = None

        self.path_array = None

    def hideTooltip( self ):
        '''Hides the tooltip and removes any ongoing timeout for the tooltip
        to be shown'''
        
        self.hide()
        self.reset()
    
    def reset( self ):
        if self.tag and not self.tag == -1:
                source_remove( self.tag )
                self.tag = None
        self.path_array = None
    
    def on_motion( self, view, event ):
        x, y = int(event.x), int(event.y)
        path_array = view.get_path_at_pos( x, y )
        if not path_array:
            # Mouse out of any user / group element
            self.reset()
            self.hide()
            return
        
        if not GROUP_TOOLTIP:
            iterator = view.get_model().get_iter( path_array[ 0 ] )
            if self.type_col < 0:
                is_user = True
            else:
                row_type = view.get_model().get_value( iterator , self.type_col )
            if row_type != 'user':
                self.hide()
                self.reset()
                return

        if self.tag and ( path_array[0] == self.path_array ):
            # TimeOut on and mouse on same element
            return
        elif self.tag and not ( path_array[0] == self.path_array ):
            # TO on and mouse has moved to another element
            self.hide()
            self.path_array = path_array[0]
            source_remove( self.tag )
            eventCoords = ( event.x_root, event.y_root, y)
            self.tag = timeout_add( DELAY, self.show_tooltip, \
                                            view, eventCoords, \
                                            path_array )
        elif ( self.tag == -1 ) and ( path_array[0] == self.path_array ):
            # Tooltip visible, no TO, mouse still in the same element
            return
        elif ( self.tag == -1 ) and not ( path_array[0] == self.path_array ):
            # Tooltip visible, no TO, mouse moves
            self.hide()
            self.path_array = path_array[0]
            eventCoords = ( event.x_root, event.y_root, y)
            self.tag = timeout_add( DELAY, self.show_tooltip, \
                                            view, eventCoords, \
                                            path_array )
        else:
            self.path_array = path_array[0]
            eventCoords = ( event.x_root, event.y_root, y)
            self.tag = timeout_add( DELAY, self.show_tooltip, \
                                            view, eventCoords, \
                                            path_array )

    def show_tooltip( self, view, origCoords, path_array ):
        self.tag = -1
        
        try:
            iterator = view.get_model().get_iter( path_array[ 0 ] )
            obj = view.get_model().get_value( iterator , self.info_col )
            mail = view.get_model().get_value( iterator , self.mail_col )
        except ValueError:
            self.reset()
            return
        
        if self.type_col < 0:
            is_user = True
        else:
            is_user = ( view.get_model().get_value( iterator , self.type_col ) == 'user' )
        
        text = self.view.getContactLabel(obj, showAlias=False, tooltip=True)
        text += '\n' + self.dataString % ( \
            obj.email, \
            self.yesNo[ str(obj.blocked) ], \
            self.yesNo[ str(obj.reverse) ], \
            self.yesNo[ str(obj.space) ], \
            self._get_last_status_since(obj.email))

        self.label.set_markup( text )

        # Sets tooltip image
        if is_user:
            contact = self.view.controller.getContact(mail)
            if contact:
                pixbuf = self.theme.getUserDisplayPicture(contact)
                self.image.set_from_pixbuf(pixbuf)
                self.image.show()
            else:
                self.image.hide()
        elif GROUP_TOOLTIP:
            # Groups get a tooltip
            # Groups have no image
            self.image.hide()
        else:
            # Groups get no tooltip
            self.tag = None
            return False

        # set the location of the tooltip
        x, y = self.computePosition( origCoords, view.window )
        self.move( x, y )
        self.show()
        return False

    def on_leave( self, view, event ):
        self.hide()
        self.reset()

    # display a border around the tooltip
    def on_expose_event( self, tooltip_window, event):
        width, height = tooltip_window.get_size()
        tooltip_window.style.paint_flat_box(tooltip_window.window, \
                                            gtk.STATE_NORMAL, gtk.SHADOW_OUT, \
                                            None, tooltip_window, 'tooltip', \
                                            0, 0, width, height)
        
    def computePosition( self, origCoords, viewWindow ):
        x_root, y_root, origY = origCoords
        currentY = viewWindow.get_pointer()[ 1 ]
        
        width, height = self.get_size()
        s_width, s_height = gtk.gdk.screen_width(), gtk.gdk.screen_height()

        x = int(x_root) - width/2
        if currentY >= origY:
            y = int(y_root) + 24
        else:
            y = int(y_root) + 6

        # check if over the screen
        if x + width > s_width:
            x = s_width - width
        elif x < 0:
            x = 0

        if y + height > s_height:
            y = y - height - 24
        elif y < 0:
            y = 0
            
        return (x, y)

    def _get_last_status_since(self, account):
        '''return a string with the format %status% since: %date% if
        the Logger plugin is available and dont return an empty result'''

        logger = self.view.controller.pluginManager.getPlugin("Logger")

        if not logger or not logger.enabled:
            return ''
            
        results = logger.get_last_status(account)
        
        if len(results) == 0:
            return ''
        
        (stamp, result) = results[0]
        return _('%(status)s since: %(time)s') % {
            'status': status.STATUS[status.MSN_TO_STATUS[result]],
            'time': time.ctime(float(stamp))}
        



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import time
import pango
import gobject
import os

import UserMenu
import GroupMenu
import TreeViewTooltips
from Parser import UserListDataType
import SmileyRenderer
import FancyAvatarRenderer

import emesenelib.common
from emesenelib.common import escape, debug

import traceback

class UserListModelTree(gtk.TreeStore):
    '''this class is the model for a tree like userlist'''
    
    def __init__(self):
        gtk.TreeStore.__init__(self, gtk.gdk.Pixbuf, 
            gobject.TYPE_PYOBJECT, str, str, gobject.TYPE_PYOBJECT, 
            bool, int, int, bool, str)
        # if its a group: None, parserdata, name, type ('group'), 
        # the group object, show pixbuf (False), offline contacts, 
        # total contacts, false, ''
        
        # if its a user:  icon, parserdata, mail, type ('user'), 
        # the contact object, show pixbuf (True), 0 if offline 1 
        # otherwise, 0, true if blocked, status
        
class UserListModelList(gtk.ListStore):
    '''this class is the model for a flat userlist'''
    
    def __init__(self):
        gtk.ListStore.__init__(self, gtk.gdk.Pixbuf, 
            gobject.TYPE_PYOBJECT, str, str, gobject.TYPE_PYOBJECT, 
            bool, int, int, bool, str)
        # if its a group: None, parserdata, name, type ('group'), 
        # the group object, show pixbuf (False), offline contacts, 
        # total contacts, false

        # if its a user:  icon, parserdata, mail, type ('user'), 
        # the contact object, show pixbuf (True), 0 if offline 1 
        # otherwise, 0, true if blocked, status
        
class UserList(gtk.TreeView):
    '''the userlist widget'''
    
    # the numbers that identify the columns of the model
    PIX = 0
    LABEL = 1
    ID = 2
    TYPE = 3
    OBJ = 4
    SHOW = 5
    OFFLINE_COUNT = 6   # for group rows
    # notoffline because if i put online you will think 
    # only on the NLN status
    NOTOFFLINE_FLAG = 6    # for contact rows
    TOTAL_COUNT = 7
    BLOCKED = 8
    STATUS = 9
    
    # type, object, path
    __gsignals__ = { 'item-selected' : (gobject.SIGNAL_RUN_LAST, \
        gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,) * 3),
    }
    
       
    def __init__(self, controller, theme, config, isTreeStore = True):
        gtk.TreeView.__init__(self)
        
        self.theme = theme
        self.config = config
        self.controller = controller
        self.unifiedParser = controller.unifiedParser
        self.smileysCache = {}

        # dont you even try to change this value!
        self.isTreeStore = isTreeStore
        
        if isTreeStore:
            self.model = UserListModelTree()
        else:
            self.model = UserListModelList()
        
        self.tooltips = TreeViewTooltips.TreeViewTooltips(self.theme, 
            self, UserList.OBJ, UserList.ID, UserList.TYPE)
            
        self.treeModelFilter = self.model.filter_new()
        self.treeModelFilter.set_visible_func(self.visible_func)
        self.set_model(self.treeModelFilter)
            
        self.filterText = ''
        # a dict with the name of the group as a key and a boolean
        # to True if its expanded and false if collapsed
        self.groupState = {}
        self.restoreGroupState()
        
        statusColor = self.personalMessageColor
        
        self.userLabel = '%s' +\
        '\n<span size="small" foreground="' + statusColor + '">%s%s</span>'
            
        # label cellrenderer
        self.crt = SmileyRenderer.SmileyRenderer()
        # icon cellrenderer
        self.crp = FancyAvatarRenderer.FancyAvatarRenderer(self.controller)
        
        column = gtk.TreeViewColumn()
        column.set_title('Icons & Text')
        column.set_expand(True)
        
        expColumn = gtk.TreeViewColumn()
        expColumn.set_max_width(16)       
        
        self.append_column(expColumn)
        self.append_column(column)
        self.set_expander_column(expColumn)
        
        column.pack_start(self.crp, False)
        column.pack_start(self.crt)
        
        column.add_attribute(self.crp, 'pixbuf', UserList.PIX)
        column.add_attribute(self.crp, 'blocked', UserList.BLOCKED)
        column.add_attribute(self.crp, 'status', UserList.STATUS)
        column.set_attributes(self.crt, markup=UserList.LABEL, 
            obj=UserList.OBJ)
        column.add_attribute(self.crp, 'visible', UserList.SHOW)
        #self.crt.set_property('ellipsize', pango.ELLIPSIZE_END)
        
        self.set_search_column(UserList.ID)
        self.set_headers_visible(False)
        
        if self.isTreeStore:
            self.TARGETS = [('TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0),
                            ('text/plain',0, 1),
                            ('TEXT', 0, 2),
                            ('STRING', 0, 3),
                            ('COMPOUND_TEXT', 0, 4),
                            ('UTF8_STRING', 0, 5)]
            self.enable_model_drag_source( gtk.gdk.BUTTON1_MASK|
                gtk.gdk.CONTROL_MASK, self.TARGETS, 
                gtk.gdk.ACTION_DEFAULT|gtk.gdk.ACTION_MOVE|
                gtk.gdk.ACTION_COPY)
            self.enable_model_drag_dest(self.TARGETS, 
                gtk.gdk.ACTION_DEFAULT )
            
            self.connect("drag_data_get", self.onDragDataGet)
            self.connect("drag_data_received", self.onDragDataReceived)
            self.connect("drag_drop", self.onDragDrop)
            
        self.connect("row_activated" , self.onRowActivated)
        self.connect("button-press-event" , self.onButtonPress)
        self.connect("row-expanded" , self.onRowExpanded)
        self.connect("row-collapsed" , self.onRowCollapsed)
        
        self.last_fill = 0
        if os.name != 'nt':  # gtk bug here?
            self.model.set_sort_func(UserList.OBJ, self.sortMethod)
            self.model.set_sort_column_id(UserList.OBJ, gtk.SORT_ASCENDING)
    
    def compareContact(self, x, y, status, type):
        def _getPriority(status, contact):
            status_priority = {
                'NLN':        0,
                'AWY':        2,
                'BSY':        6,
                'BRB':        3,
                'PHN':        4,
                'LUN':        5,
                'HDN':        7,
                'IDL':        1,
                'FLN':        8,
            }
            if not status:
                if contact.status == 'FLN':
                    return 8
                else:
                    return 0
            else:
                return status_priority[contact.status]
            
        if type:
            value = lambda x: x.email
        else:
            value = lambda x: x.nick
            
        if _getPriority(status, x) > _getPriority(status,y):
            return 1
        elif _getPriority(status, x) < _getPriority(status,y):
            return -1
        else:
            if value(x) == value(y):
                return 0
            elif value(x) > value(y):
                return 1
            else:
                return -1
         
    def sortMethod(self, treemodel, iter1, iter2, user_data=None):
        ''' method used for sorting the userlist'''

        status = self.sortNickGroupByStatusPriority
        type = self.sortNickGroupByContact
        obj1 = self.model[iter1][UserList.OBJ]
        obj2 = self.model[iter2][UserList.OBJ]
        
        if self.model[iter1][UserList.TYPE] == 'group' or \
            self.model[iter2][UserList.TYPE] == 'group':
            return -1
        try:
            return  self.compareContact(obj1, obj2, status, type)
        except:
            return -1
 
    def onDragDataGet(self, treeview, context, selection, target_id, etime):
        # TODO: what does the 8 magic number mean?
        (rowType, contact) = self.getSelected()
        if rowType == 'user':
            if target_id == 0:
                selection.set('STRING', 8, contact.email + '|' + self.getGroupSelected().name)
            else:
                selection.set('STRING', 8, contact.nick + ' <' + contact.email + '>')
        else:
            if target_id == 0:
                selection.set('STRING', 8, '')
            else:
                selection.set('STRING', 8, self.getGroupSelected().name)
    
    def onDragDrop(self, widget, context, x, y, time):
        widget.drag_get_data( context, context.targets[0], time )
        return True
    
    def onDragDataReceived(self, treeview, context, x, y, selection, 
        info, etime):
        # don't move contacts when ordered by status    
        if self.orderByStatus or selection.data == '':
            context.finish( False, False, etime )
            return
        if info != 0:
            context.finish( False, False, etime )
            return
        
        model = treeview.get_model()
        mail, group = selection.data.split('|')
        drop_info = treeview.get_dest_row_at_pos(x, y)
        
        if not drop_info:
            context.finish( False, False, etime )
            return
        
        path, position = drop_info
        iter = model.get_iter(path)
        type, object = self.getTupleByIter(iter)
        
        if type == 'group':
            destGroup = object
        elif type == 'user':
            destGroup = self.getGroupSelected(iter)
        
        if context.action == gtk.gdk.ACTION_COPY:
            self.controller.contacts.add_to_group(mail, destGroup.name)
        else:
            self.controller.contacts.move_to_group(mail, group, 
                destGroup.name)
        
        context.finish( True, False, etime )
    
    # -- CONFIG PROPERTIES -- #
    def getSortNickGroupByStatusPriority(self):
        return self.config.user['sortNickGroupByStatusPriority']
    sortNickGroupByStatusPriority = property(fget=getSortNickGroupByStatusPriority)
    
    def getSortNickGroupByContact(self):
        return self.config.user['sortNickGroupByContact']
    sortNickGroupByContact = property(fget=getSortNickGroupByContact)
    
    def getShowByNick(self):
        return self.config.user['showByNick']
    showByNick = property(fget=getShowByNick)

    def getOrderByStatus(self):
        return self.config.user['orderByStatus']
    orderByStatus = property(fget=getOrderByStatus)

    def getShowOffline(self):
        return self.config.user['showOffline']
    showOffline = property(fget=getShowOffline)
    
    def getShowEmptyGroups(self):
        return self.config.user['showEmptyGroups']
    showEmptyGroups = property(fget=getShowEmptyGroups)

    def getShowCountContact(self):
        return self.config.user['showCountContact']
    showCountContact = property(fget=getShowCountContact)

    def getAvatarsInUserList(self):
        return self.config.user['avatarsInUserList']
    avatarsInUserList = property(fget=getAvatarsInUserList)

    def getPersonalMessageColor(self):
        return self.config.user['personalMessageColor']
    personalMessageColor = property(fget=getPersonalMessageColor)
        
    def getSmallIcons(self):
        return self.config.user['smallIcons']
    smallIcons = property(fget=getSmallIcons)
    
    def getParseSmilies(self):
        return self.config.user['parseSmilies']
    parseSmilies = property(fget=getParseSmilies)

    def getUserListAvatarSize(self):
        return self.config.user['userListAvatarSize']
    avatarSize = property(fget=getUserListAvatarSize)
    
    # -- FILTER STUFF -- #
    
    def visible_func(self, model, iter):
        obj = model[iter][UserList.OBJ]
        text = self.filterText.lower()
        if text == '' and obj and not self.showOffline and \
           model[iter][UserList.TYPE] == 'user' and obj.status == 'FLN':
            return False
        elif model[iter][UserList.TYPE] == 'group':
            totalCount = model[iter][UserList.TOTAL_COUNT]
            offlineCount = model[iter][UserList.OFFLINE_COUNT]
            if totalCount == offlineCount:
                if (self.showOffline and totalCount > 0) or \
                   self.showEmptyGroups or text != '':
                    return True
                else:
                    return False
            return True
        elif text == '':
            return True
        elif obj and \
             (obj.email.lower().find(text) != -1 or \
              obj.nick.lower().find(text) != -1 or \
              obj.alias.lower().find(text) != -1):
            return True

        return False

    def refilter(self):
        self.treeModelFilter.refilter()
        self.expandExpandedGroups()
        
    def setFilterText(self, text):
        self.filterText = text
        self.treeModelFilter.refilter()
        self.expandExpandedGroups()

    # -- OTHER FUNCTIONS -- #
    # TODO: group them

    def expandExpandedGroups(self):
        '''iterate over the groups and expand the ones that have True on
        self.groupState'''

        if self.isTreeStore:
            for gRow in self.treeModelFilter:
                try:
                    obj = gRow[UserList.OBJ]
                except:
                    return
                if not (obj.id in self.groupState and
                        self.groupState[obj.id] == False) or \
                   self.orderByStatus:

                    self.expand_row(gRow.path, False)

    def sortGroupByName(self, groupDict):
        '''this method return a list of groupNames sorted according the 
        configuration'''

        l = groupDict.values() 
        l.sort( lambda x, y: cmp(x.name, y.name) )
        return l
        
    def sortByStatus(self, groupDict):
        '''receive a groupDict and return a dict of contacts sorted by status'''
        
        online = emesenelib.ContactData.Group(_('Online'))
        #offline = emesenelib.ContactData.Group(_('Offline'))
        for group in groupDict.values():
            for user in group.users.values():
                if False: # user.status == 'FLN':
                    offline.setUser(user.email, user)
                else:
                    online.setUser(user.email, user)
                
        #return [online, offline]
        return [online]
        
    def getSortedContact(self, group, status , type):
        ''' return a list of contact by status and/or type '''
        contactIds = group.users.values()
        contactIds.sort( lambda x,y: self.compareContact(x, y, status, type))
        return  [x.email for x in contactIds]
    
    def getUserSelected(self):
        '''return the user name if a user is selected or an empty string
        if no user is selected'''
        
        (rowType, contact) = self.getSelected()
        
        if rowType == 'user':
            return contact.email
        else:
            return ''
        
    def groupIsSmaller(self, group1, group2):
        '''return True if the group1 < group2 according to the criteria 
        specified by the preferences'''
        
        return group1.name < group2.name
        
    def contactIsSmaller(self, contact1, contact2):
        '''return True if the contact1 < contact2 according to the criteria 
        specified by the preferences'''
        
        return contact1.email < contact2.email
        
    def allContactsOffline(self, contacts):
        '''return True if all contacts are offline'''
        
        return len([x for x in contacts if x.status != 'FLN']) == 0
       
    def add(self, values, iterator=None):
        '''add an item to the model'''
        if self.isTreeStore:
            return self.model.append(iterator, values)
        else:
            return self.model.append(values)
        
    def fill(self, groupDict=None, _force=False):
        '''clear and fill the user list, groupDict is a dict containing
        emesenelib.ContactData.Group objects as value and his name as a key'''

        if groupDict is None:
            groupDict = self.controller.msn.contactManager.groups

        self.set_model(None)
        #traceback.print_stack()
        self.model.clear()

        # refresh crp dimention; fill is called when
        # theme prefrences (smallIcons) change, set crp dimention here
        if self.smallIcons:
            self.crp.set_property('dimention', self.avatarSize / 2)
        else:
            self.crp.set_property('dimention', self.avatarSize) 

        self.userLabel = '%s\n<span size="small" foreground="' + \
            self.personalMessageColor + '">%(status)s%s</span>'
        
        # self.showEmptyGroups ask to config, so we use a temp value
        showEmptyGroups = self.showEmptyGroups
        showOffline = self.showOffline
        
        if self.orderByStatus:
            sortedGroups = self.sortByStatus(groupDict)
        else:
            sortedGroups = self.sortGroupByName(groupDict)
            
        for group in sortedGroups:
            contacts = self.getSortedContact(group,self.sortNickGroupByStatusPriority,self.sortNickGroupByContact)
            
            groupIter = self.add(self.getGroupRow(group))

            offlineCount = 0
            totalCount = 0

            for contactId in contacts:
                contact = group.users[contactId]
                totalCount += 1
                if contact.status == 'FLN':
                    offlineCount +=1
                # groupIter is not used id we are a list
                self.add(self.getContactRow(contact), groupIter)
                
            self.model.set_value(groupIter, UserList.TOTAL_COUNT, totalCount)
            self.model.set_value(groupIter, UserList.OFFLINE_COUNT, offlineCount)

        self.set_model(self.treeModelFilter)
        self.expandExpandedGroups()

    def getMenuData(self):
        '''Returns useful data for building menus in the format
        (<selected item type>, [[<item name>], <item's group>], <item id>)'''

        data = self.getSelected()

        if data:
            (rowType, obj) = data
            if rowType == 'user':
                return (rowType, obj.email, self.getGroupSelected().name, obj.id)
            elif rowType == 'group':
                return (rowType, obj.name, obj.id)
        else:
            return ('',)

    def getGroupLabel(self, group):
        '''return the pango string to format the group'''
        if self.showCountContact:
            return '<b>%s ( %d/%d )</b>' % (escape(group.name), \
                group.getOnlineUsersNumber(), group.getSize())
        else:
            return '<b>%s</b>' % escape(group.name)

    def getGroupRow(self, group):
        '''return the list that contain all the fields to add it to the model'''
        return [None, self.getGroupLabel(group), group.name, 'group', group, False, 0, 0, False, '']
    
    def getContactLabel(self, contact, showAlias=True, tooltip=False):
        '''returns a smileyrenderer list (if tooltip=False) or
        a plain pango string (if tooltip=True)'''
        status = ''
        
        if contact.status != 'NLN':
            index = self.controller.status_ordered[0].index(contact.status)
            # you fail at i18n
            status = '(' + self.unifiedParser.getParser( \
                _(self.controller.status_ordered[2][index])).get() + ')' + \
                (contact.personalMessage and ' - ')
        
        if contact.alias != '' and showAlias:
            label = contact.alias    
        elif self.showByNick:
            label = contact.nick
        else:
            label = contact.email
        psm = contact.personalMessage

        hasSmilies = self.parseSmilies
        smallIcons = self.smallIcons

        if smallIcons and not tooltip:
            if not psm and not status:
                template = self.userLabel.replace("\n", '')
            else:
                template = self.userLabel.replace("\n", " - ")
        else:
            template = self.userLabel

        template = template.replace("%(status)s", status)
        
        # TODO: this shows how the current parser design sucks
        if tooltip:
            # YES! YOU'VE GUESSED! THIS IS A HACK!
            text = label.replace('\n', '') + '\n' + psm.replace('\n', '')
            text = self.unifiedParser.getParser(text).get().split('\n')

            try:
                label = text[0]
                psm = text[1]
            except IndexError:
                pass
            return template % (label, psm)
        else:
            tuple = (template, label, psm)
            
            #send to parser
            parser = self.unifiedParser.getParser(tuple, UserListDataType)
            return parser.get(hasSmilies, self.smileysCache)
          
    def getContactImage(self, contact):    
        '''return the image that will be displayed on the userlist'''
        fallback = 'online'
        if self.avatarsInUserList and \
           self.theme.hasUserDisplayPicture(contact):
            return self.theme.getUserDisplayPicture(contact, \
                self.avatarSize, self.avatarSize)
        elif contact.status == 'FLN': 
            if contact.mobile:
                fallback = 'mobile'
            else:
                fallback = 'offline'

        return self.theme.getImage(fallback)
          
    def getContactRow(self, contact):
        '''return the list that contain all the fields to add it to the model'''
        notofflineFlag = 0 # he is offline

        if contact.status != 'FLN':
            notofflineFlag = 1 # he is not offline
        return [self.getContactImage(contact), self.getContactLabel(contact), contact.email, 'user', contact, True, notofflineFlag, 0,
                contact.blocked, contact.status]
          
    def getGroupIter(self, group, add=False):
        '''try to find a group and return the iter, iff not found and add == True
        add it and return the iter, if add==false and not found return None'''
        
        if self.isTreeStore:
            for gRow in self.treeModelFilter:
                obj = gRow[UserList.OBJ]
                if obj.name == group.name:
                    return gRow.iter
        else:
            for row in self.treeModelFilter:
                obj = row[UserList.OBJ]
                if row[UserList.TYPE] == 'group' and obj.name == group.name:
                    return row.iter
        
        if add:
            return self.addGroup(group)            

        return None
          
    def getTupleByIter(self, iterator):
        '''returns a tuple (type, object) corresponding to the given iterator
        type is a string, it can be either 'group' or 'user'
        object is a group or user instance, depending on type'''
        return (self.treeModelFilter[iterator][UserList.TYPE],
                self.treeModelFilter[iterator][UserList.OBJ])
          
    def getSelected(self):
        '''return a tuple containing the type ("group" or "user") and the object
        or None'''
        
        rows = self.get_selection().get_selected_rows()[1]
        iterator = None

        if len(rows) == 1:
            iterator = rows[0]

        if iterator is None:
            debug('invalid iter')
            return None
            
        return self.getTupleByIter(iterator)
        
    def getGroupSelected(self, iterator=None):
        '''if a group is selected return the group object, if a user is
        selected, return the group in wich its contained'''
        
        if iterator == None:
            iterator = self.get_selection().get_selected()[1]

        objType = self.treeModelFilter[iterator][UserList.TYPE]
        if objType == 'user':
            return self.treeModelFilter[iterator].parent[UserList.OBJ]
        elif objType == 'group':
            return self.treeModelFilter[iterator][UserList.OBJ]
        
    def updateGroup(self, oldGroup, group):
        '''update the values of group, we use oldGroup because the name could
        have changed, the users inside the group are not modified'''
        # TODO: it erase the children nodes
        for row in self.model:
            obj = row[UserList.OBJ]
            if row[UserList.TYPE] == 'group' and obj.name == oldGroup.name:
                self.model.set_value(row.iter, UserList.LABEL, self.getGroupLabel(group))
                self.model.set_value(row.iter, UserList.OBJ, group)
                break
    
    def updateGroupNum(self,contact):
        for row in self.model:
            obj = row[UserList.OBJ]
            if row[UserList.TYPE] == 'group' and obj.getUser(contact.email) != None :
                obj.setUser(contact.email,contact)
                self.model.set_value(row.iter, UserList.LABEL, self.getGroupLabel(obj))
    
    def updateContact(self, contact):
        '''update the values of a contact, dont use old because the mail cant
        change'''
        if not contact:
            print 'Contact is None (UserList:703)'
            return

        doRefilter = False
        if self.isTreeStore:
            for gRow in self.model:
                for row in gRow.iterchildren():
                    if row[UserList.TYPE] == 'user' and \
                       row[UserList.OBJ].email == contact.email:
                        doRefilter = self._updateContactRow(contact, row, gRow)
        else:
            for row in self.model:
                obj = row[UserList.OBJ]
                if row[UserList.TYPE] == 'user' and obj.email == contact.email:
                    oldStatus = obj.status
                    doRefilter = self._updateContactRow(contact, row)

        if doRefilter:
            if False: #self.orderByStatus:
                #print '===fill==='
                self.fill(self.controller.msn.contactManager.groups)
            else:
                self.refilter()
        self.updateGroupNum(contact)
    
    def _updateContactRow(self, contact, row, gRow=None):
        '''updates a userlist row with new data, returns
        bool that determines if refilter is needed or not'''
        row[UserList.PIX] = self.getContactImage(contact)
        row[UserList.LABEL] = self.getContactLabel(contact)
        row[UserList.OBJ] = contact
        row[UserList.BLOCKED] = contact.blocked
        row[UserList.STATUS] = contact.status
        if gRow == None:
            return False

        # if he is offline and he wasnt offline until now
        if contact.status == 'FLN' and row[UserList.NOTOFFLINE_FLAG] == 1:
            # set the is offline flas to 0 (he is offline)
            row[UserList.NOTOFFLINE_FLAG] = 0

            # increment the offline count by 1
            self.model.set_value(gRow.iter, UserList.OFFLINE_COUNT, \
                gRow[UserList.OFFLINE_COUNT] + 1)
            
            # if all now, are offline refilter
            online = gRow[UserList.TOTAL_COUNT] - gRow[UserList.OFFLINE_COUNT]
            if online == 0 or self.orderByStatus:
                return True
            
        # if he wasnt offline and the flag says that he was offline
        elif contact.status != 'FLN' and row[UserList.NOTOFFLINE_FLAG] == 0:
            # set that he is not offline
            row[UserList.NOTOFFLINE_FLAG] = 1

            # decrease the offline count by 1
            self.model.set_value(gRow.iter, UserList.OFFLINE_COUNT, \
                gRow[UserList.OFFLINE_COUNT] - 1)

            # if only one contact is online in the group we refresh
            online = gRow[UserList.TOTAL_COUNT] - gRow[UserList.OFFLINE_COUNT]
            if online == 1 or self.orderByStatus:
                return True

        return False

    def addGroup(self, group):
        '''add a group to the list, return the gtk.TreeIter'''
        
        for row in self.treeModelFilter:
            obj = row[UserList.OBJ]
            if row[UserList.TYPE] == 'group' and self.groupIsSmaller(group, obj):
                if self.isTreeStore:
                    return self.model.insert_before(None, row.iter, self.getGroupRow(group))
                else:
                    return self.model.insert_before(row.iter, self.getGroupRow(group))
        
        if self.isTreeStore:
            return self.model.append(None, self.getGroupRow(group))
        else:
            return self.model.append(self.getGroupRow(group))
    
    def addContact(self, group, contact):
        '''add a contact to the given group, if this is not on the list, its added'''
        
        gRow = self.treeModelFilter[self.getGroupIter(group, True)]
        
        if self.isTreeStore:
            for row in gRow.iterchildren():
                obj = row[UserList.OBJ]
                if row[UserList.TYPE] == 'user' and self.contactIsSmaller(contact, obj):
                    return self.model.insert_before(gRow.iter, row.iter, self.getContactRow(contact))
        else:
            treeModelRow = self.treeModelFilter[gRow.iter]
            treeModelRow = treeModelRow.next
            row = self.treeModelFilter[treeModelRow.iter]
            
            while treeModelRow:
                row = self.treeModelFilter[treeModelRow.iter]
                obj = row[UserList.OBJ]
                # if we reached the next group we add it before
                if row[UserList.TYPE] == 'group':
                    break         
                if row[UserList.TYPE] == 'user' and self.contactIsSmaller(contact, obj):
                    break
                    
                treeModelRow = treeModelRow.next
           
            return self.model.insert_before(row.iter, self.getContactRow(contact))
                    
    # ------------------------ Callbacks -------------------------------------
    
    def onRowActivated(self, treeview, path, view_column):
        tup = self.getSelected()
        objType = obj = None
        if tup:
            objType, obj = tup
        self.emit('item-selected', objType, obj, path)
        
    def onButtonPress(self, treeview, event):
        if event.button == 3 and self.isTreeStore:
            paths = treeview.get_path_at_pos(int(event.x), int(event.y))
            
            if paths == None:
                debug('invalid path')
            elif len(paths) > 0:
                iterator = self.treeModelFilter.get_iter(paths[0])
                (rowType,obj) = self.getTupleByIter(iterator)

                self.tooltips.hideTooltip()
                
                if rowType == 'user':
                    menu = UserMenu.UserMenu(self.controller, obj, \
                        self.getGroupSelected(iterator))
                    menu.popup(None, None, None, event.button, event.time)
                elif rowType == 'group' and not self.orderByStatus:
                    menu = GroupMenu.GroupMenu(self.controller , obj)
                    menu.popup(None, None, None, event.button, event.time)
            else:
                debug('empty paths?')

    def onRowExpanded(self,treeview, iterator, path):
        obj = self.treeModelFilter[path][UserList.OBJ]
        self.groupState[obj.id] = True
        self.saveGroupState()

    def onRowCollapsed(self,treeview, iterator, path):
        obj = self.treeModelFilter[path][4]
        self.groupState[obj.id] = False
        self.saveGroupState()

    def restoreGroupState(self):
        '''restore self.groupState'''
        value = self.config.user['collapsedGroups']
        collapsed = value.split(',')
        for gid in collapsed:
            self.groupState[gid] = False

    def saveGroupState(self):
        '''save ids of collapsed groups'''
        collapsed = []
        for gid in self.groupState:
            if not self.groupState[gid]:
                collapsed.append(gid)

        self.config.user['collapsedGroups'] = ','.join(collapsed)

gobject.type_register(UserList)




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import Plugin

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = _('Save the personal message between sessions.')
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = _('Personal Message')
        self.name = 'PersonalMessage'
        
        self.config = controller.config
        self.config.readPluginConfig(self.name)
        
        self.enabled = False
        
    def start( self ):
        '''start the plugin'''

        self.enabled = True
        self.msn.changePersonalMessage(self.config.getPluginValue(self.name, 'pm', ''))
        self.msn.connect( 'self-personal-message-changed', self.action )
        
    def stop( self ):    
        '''stop the plugin'''
        
        self.enabled = False
        
    def action( self, msnp, user, pm ):
        if self.enabled:
            self.config.setPluginValue(self.name, 'pm', pm)
    
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        return ( True, 'Ok' )
